# Copyright 2013, Mirantis Inc
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from horizon.workflows import views
from openstack_dashboard import api
from openstack_dashboard.test import helpers

from neutron_vpnaas_dashboard.api import vpn as api_vpn
from neutron_vpnaas_dashboard.dashboards.project.vpn import workflows
from neutron_vpnaas_dashboard.test import helpers as test


class VPNTests(test.TestCase):

    class AttributeDict(dict):
        def __getattr__(self, attr):
            return self[attr]

        def __setattr__(self, attr, value):
            self[attr] = value

    DASHBOARD = 'project'
    INDEX_URL = reverse('horizon:%s:vpn:index' % DASHBOARD)

    ADDIKEPOLICY_PATH = 'horizon:%s:vpn:addikepolicy' % DASHBOARD
    ADDIPSECPOLICY_PATH = 'horizon:%s:vpn:addipsecpolicy' % DASHBOARD
    ADDVPNSERVICE_PATH = 'horizon:%s:vpn:addvpnservice' % DASHBOARD
    ADDENDPOINTGROUP_PATH = 'horizon:%s:vpn:addendpointgroup' % DASHBOARD
    ADDVPNCONNECTION_PATH = 'horizon:%s:vpn:addipsecsiteconnection' % DASHBOARD

    IKEPOLICY_DETAIL_PATH = 'horizon:%s:vpn:ikepolicydetails' % DASHBOARD
    IPSECPOLICY_DETAIL_PATH = 'horizon:%s:vpn:ipsecpolicydetails' % DASHBOARD
    VPNSERVICE_DETAIL_PATH = 'horizon:%s:vpn:vpnservicedetails' % DASHBOARD
    ENDPOINTGROUP_DETAIL_PATH = 'horizon:%s:vpn:endpointgroupdetails' %\
        DASHBOARD
    VPNCONNECTION_DETAIL_PATH = 'horizon:%s:vpn:ipsecsiteconnectiondetails' %\
        DASHBOARD

    UPDATEIKEPOLICY_PATH = 'horizon:%s:vpn:update_ikepolicy' % DASHBOARD
    UPDATEIPSECPOLICY_PATH = 'horizon:%s:vpn:update_ipsecpolicy' % DASHBOARD
    UPDATEVPNSERVICE_PATH = 'horizon:%s:vpn:update_vpnservice' % DASHBOARD
    UPDATEENDPOINTGROUP_PATH = 'horizon:%s:vpn:update_endpointgroup' %\
        DASHBOARD
    UPDATEVPNCONNECTION_PATH = 'horizon:%s:vpn:update_ipsecsiteconnection' %\
        DASHBOARD

    def setup_mocks(self):
        self.mock_vpnservice_list.return_value = self.vpnservices.list()
        self.mock_endpointgroup_list.return_value = self.endpointgroups.list()
        self.mock_ikepolicy_list.return_value = self.ikepolicies.list()
        self.mock_ipsecpolicy_list.return_value = self.ipsecpolicies.list()
        self.mock_ipsecsiteconnection_list.return_value = \
            self.ipsecsiteconnections.list()

    def setup_mocks_with_exception(self):
        self.mock_vpnservice_list.side_effect = self.exceptions.neutron
        self.mock_endpointgroup_list.side_effect = self.exceptions.neutron
        self.mock_ikepolicy_list.side_effect = self.exceptions.neutron
        self.mock_ipsecpolicy_list.side_effect = self.exceptions.neutron
        self.mock_ipsecsiteconnection_list.side_effect = \
            self.exceptions.neutron

    def check_mocks(self):
        self.mock_vpnservice_list.assert_called_once_with(
            helpers.IsHttpRequest(), tenant_id=self.tenant.id)
        self.mock_endpointgroup_list.assert_called_once_with(
            helpers.IsHttpRequest(), tenant_id=self.tenant.id)
        self.mock_ikepolicy_list.assert_called_once_with(
            helpers.IsHttpRequest(), tenant_id=self.tenant.id)
        self.mock_ipsecpolicy_list.assert_called_once_with(
            helpers.IsHttpRequest(), tenant_id=self.tenant.id)
        self.mock_ipsecsiteconnection_list.assert_called_once_with(
            helpers.IsHttpRequest(), tenant_id=self.tenant.id)

    @helpers.create_mocks({api_vpn: ('ikepolicy_list', 'ipsecpolicy_list',
                                     'vpnservice_list', 'endpointgroup_list',
                                     'ipsecsiteconnection_list')})
    def test_index_vpnservices(self):
        self.setup_mocks()

        res = self.client.get(self.INDEX_URL + '?tab=vpntabs__vpnservices')

        self.assertTemplateUsed(res, '%s/vpn/index.html'
                                % self.DASHBOARD)
        self.assertTemplateUsed(res, 'horizon/common/_detail_table.html')
        self.assertEqual(len(res.context['vpnservicestable_table'].data),
                         len(self.vpnservices.list()))
        self.check_mocks()

    @helpers.create_mocks({api_vpn: ('ikepolicy_list', 'ipsecpolicy_list',
                                     'vpnservice_list', 'endpointgroup_list',
                                     'ipsecsiteconnection_list')})
    def test_index_endpointgroups(self):
        self.setup_mocks()

        res = self.client.get(self.INDEX_URL + '?tab=vpntabs__endpointgroups')

        self.assertTemplateUsed(res, '%s/vpn/index.html'
                                % self.DASHBOARD)
        self.assertTemplateUsed(res, 'horizon/common/_detail_table.html')
        self.assertEqual(len(res.context['endpointgroupstable_table'].data),
                         len(self.endpointgroups.list()))
        self.check_mocks()

    @helpers.create_mocks({api_vpn: ('ikepolicy_list', 'ipsecpolicy_list',
                                     'vpnservice_list', 'endpointgroup_list',
                                     'ipsecsiteconnection_list')})
    def test_index_ikepolicies(self):
        self.setup_mocks()

        res = self.client.get(self.INDEX_URL + '?tab=vpntabs__ikepolicies')

        self.assertTemplateUsed(res, '%s/vpn/index.html'
                                % self.DASHBOARD)
        self.assertTemplateUsed(res, 'horizon/common/_detail_table.html')
        self.assertEqual(len(res.context['ikepoliciestable_table'].data),
                         len(self.ikepolicies.list()))
        self.check_mocks()

    @helpers.create_mocks({api_vpn: ('ikepolicy_list', 'ipsecpolicy_list',
                                     'vpnservice_list', 'endpointgroup_list',
                                     'ipsecsiteconnection_list')})
    def test_index_ipsecpolicies(self):
        self.setup_mocks()

        res = self.client.get(self.INDEX_URL + '?tab=vpntabs__ipsecpolicies')

        self.assertTemplateUsed(res, '%s/vpn/index.html'
                                % self.DASHBOARD)
        self.assertTemplateUsed(res, 'horizon/common/_detail_table.html')
        self.assertEqual(len(res.context['ipsecpoliciestable_table'].data),
                         len(self.ipsecpolicies.list()))
        self.check_mocks()

    @helpers.create_mocks({api_vpn: ('ikepolicy_list', 'ipsecpolicy_list',
                                     'vpnservice_list', 'endpointgroup_list',
                                     'ipsecsiteconnection_list')})
    def test_index_ipsecsiteconnections(self):
        self.setup_mocks()

        res = self.client.get(
            self.INDEX_URL + '?tab=vpntabs__ipsecsiteconnections')

        self.assertTemplateUsed(res, '%s/vpn/index.html'
                                % self.DASHBOARD)
        self.assertTemplateUsed(res, 'horizon/common/_detail_table.html')
        self.assertEqual(
            len(res.context['ipsecsiteconnectionstable_table'].data),
            len(self.ipsecsiteconnections.list()))
        self.check_mocks()

    @helpers.create_mocks({api_vpn: ('ikepolicy_list', 'ipsecpolicy_list',
                                     'vpnservice_list', 'endpointgroup_list',
                                     'ipsecsiteconnection_list')})
    def test_index_exception_vpnservices(self):
        self.setup_mocks_with_exception()

        res = self.client.get(self.INDEX_URL)

        self.assertTemplateUsed(res, '%s/vpn/index.html'
                                % self.DASHBOARD)
        self.assertTemplateUsed(res,
                                'horizon/common/_detail_table.html')
        self.assertEqual(len(res.context['table'].data), 0)
        self.check_mocks()

    @helpers.create_mocks({api_vpn: ('ikepolicy_list', 'ipsecpolicy_list',
                                     'vpnservice_list', 'endpointgroup_list',
                                     'ipsecsiteconnection_list')})
    def test_index_exception_endpointgroups(self):
        self.setup_mocks_with_exception()

        res = self.client.get(self.INDEX_URL + '?tab=vpntabs__endpointgroups')

        self.assertTemplateUsed(res, '%s/vpn/index.html'
                                % self.DASHBOARD)
        self.assertTemplateUsed(res,
                                'horizon/common/_detail_table.html')
        self.assertEqual(len(res.context['table'].data), 0)
        self.check_mocks()

    @helpers.create_mocks({api_vpn: ('ikepolicy_list', 'ipsecpolicy_list',
                                     'vpnservice_list', 'endpointgroup_list',
                                     'ipsecsiteconnection_list')})
    def test_index_exception_ikepolicies(self):
        self.setup_mocks_with_exception()

        res = self.client.get(self.INDEX_URL + '?tab=vpntabs__ikepolicies')

        self.assertTemplateUsed(res, '%s/vpn/index.html'
                                % self.DASHBOARD)
        self.assertTemplateUsed(res,
                                'horizon/common/_detail_table.html')
        self.assertEqual(len(res.context['table'].data), 0)
        self.check_mocks()

    @helpers.create_mocks({api_vpn: ('ikepolicy_list', 'ipsecpolicy_list',
                                     'vpnservice_list', 'endpointgroup_list',
                                     'ipsecsiteconnection_list')})
    def test_index_exception_ipsecpolicies(self):
        self.setup_mocks_with_exception()

        res = self.client.get(self.INDEX_URL + '?tab=vpntabs__ipsecpolicies')

        self.assertTemplateUsed(res, '%s/vpn/index.html'
                                % self.DASHBOARD)
        self.assertTemplateUsed(res,
                                'horizon/common/_detail_table.html')
        self.assertEqual(len(res.context['table'].data), 0)
        self.check_mocks()

    @helpers.create_mocks({api_vpn: ('ikepolicy_list', 'ipsecpolicy_list',
                                     'vpnservice_list', 'endpointgroup_list',
                                     'ipsecsiteconnection_list')})
    def test_index_exception_ipsecsiteconnections(self):
        self.setup_mocks_with_exception()

        res = self.client.get(
            self.INDEX_URL + '?tab=vpntabs__ipsecsiteconnections')

        self.assertTemplateUsed(res, '%s/vpn/index.html'
                                % self.DASHBOARD)
        self.assertTemplateUsed(res,
                                'horizon/common/_detail_table.html')
        self.assertEqual(len(res.context['table'].data), 0)
        self.check_mocks()

    @helpers.create_mocks({api.neutron: ('network_list_for_tenant',
                                         'router_list')})
    def test_add_vpnservice_get(self):
        networks = [{'subnets': [self.subnets.first(), ]}, ]
        routers = self.routers.list()

        self.mock_network_list_for_tenant.return_value = networks
        self.mock_router_list.return_value = routers

        res = self.client.get(reverse(self.ADDVPNSERVICE_PATH))

        workflow = res.context['workflow']
        self.assertTemplateUsed(res, views.WorkflowView.template_name)
        self.assertEqual(workflow.name, workflows.AddVPNService.name)

        expected_objs = ['<AddVPNServiceStep: addvpnserviceaction>', ]
        steps = [repr(step) for step in workflow.steps]
        self.assertQuerySetEqual(steps, expected_objs)

        self.mock_network_list_for_tenant.assert_called_once_with(
            helpers.IsHttpRequest(), self.tenant.id)
        self.mock_router_list.assert_called_once_with(
            helpers.IsHttpRequest(), tenant_id=self.tenant.id)

    @helpers.create_mocks({api.neutron: ('router_list',
                                         'network_list_for_tenant'),
                           api_vpn: ('vpnservice_create', )})
    def test_add_vpnservice_post(self):
        vpnservice = self.vpnservices.first()
        networks = [{'subnets': [self.subnets.first(), ]}, ]
        routers = self.routers.list()

        self.mock_network_list_for_tenant.return_value = networks
        self.mock_router_list.return_value = routers

        form_data = {'name': vpnservice['name'],
                     'description': vpnservice['description'],
                     'subnet_id': vpnservice['subnet_id'],
                     'router_id': vpnservice['router_id'],
                     'admin_state_up': vpnservice['admin_state_up']}

        self.mock_vpnservice_create.return_value = vpnservice

        res = self.client.post(reverse(self.ADDVPNSERVICE_PATH), form_data)

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, str(self.INDEX_URL))

        self.mock_network_list_for_tenant.assert_called_once_with(
            helpers.IsHttpRequest(), self.tenant.id)
        self.mock_router_list.assert_called_once_with(
            helpers.IsHttpRequest(), tenant_id=self.tenant.id)
        self.mock_vpnservice_create.assert_called_once_with(
            helpers.IsHttpRequest(), **form_data)

    @helpers.create_mocks({api.neutron: ('router_list',
                                         'network_list_for_tenant')})
    def test_add_vpnservice_post_error(self):
        vpnservice = self.vpnservices.first()
        networks = [{'subnets': [self.subnets.first(), ]}, ]
        routers = self.routers.list()

        self.mock_network_list_for_tenant.return_value = networks
        self.mock_router_list.return_value = routers

        form_data = {'name': vpnservice['name'],
                     'description': vpnservice['description'],
                     'subnet_id': '',
                     'router_id': '',
                     'admin_state_up': vpnservice['admin_state_up']}

        res = self.client.post(reverse(self.ADDVPNSERVICE_PATH), form_data)

        self.assertFormErrors(res, 1)

        self.mock_network_list_for_tenant.assert_called_once_with(
            helpers.IsHttpRequest(), self.tenant.id)
        self.mock_router_list.assert_called_once_with(
            helpers.IsHttpRequest(), tenant_id=self.tenant.id)

    @helpers.create_mocks({api.neutron: ('network_list_for_tenant', )})
    def test_add_endpointgroup_get(self):
        networks = [{'subnets': [self.subnets.first(), ]}, ]

        self.mock_network_list_for_tenant.return_value = networks

        res = self.client.get(reverse(self.ADDENDPOINTGROUP_PATH))

        workflow = res.context['workflow']
        self.assertTemplateUsed(res, views.WorkflowView.template_name)
        self.assertEqual(workflow.name, workflows.AddEndpointGroup.name)

        expected_objs = ['<AddEndpointGroupStep: addendpointgroupaction>', ]
        steps = [repr(step) for step in workflow.steps]
        self.assertQuerySetEqual(steps, expected_objs)
        self.mock_network_list_for_tenant.assert_called_once_with(
            helpers.IsHttpRequest(), self.tenant.id)

    @helpers.create_mocks({api.neutron: ('network_list_for_tenant', ),
                           api_vpn: ('endpointgroup_create', )})
    def test_add_endpointgroup_post(self):
        endpointgroup = self.endpointgroups.first()
        networks = [{'subnets': [self.subnets.first(), ]}, ]

        self.mock_network_list_for_tenant.return_value = networks

        form_data = {'name': endpointgroup['name'],
                     'description': endpointgroup['description'],
                     'subnets': endpointgroup['endpoints'],
                     'type': endpointgroup['type']}

        self.mock_endpointgroup_create.return_value = endpointgroup

        res = self.client.post(reverse(self.ADDENDPOINTGROUP_PATH), form_data)

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, str(self.INDEX_URL))

        self.mock_network_list_for_tenant.assert_called_once_with(
            helpers.IsHttpRequest(), self.tenant.id)
        data = form_data.copy()
        data['endpoints'] = data['subnets']
        del data['subnets']
        self.mock_endpointgroup_create.assert_called_once_with(
            helpers.IsHttpRequest(), **data)

    @helpers.create_mocks({api.neutron: ('network_list_for_tenant', )})
    def test_add_endpointgroup_post_error(self):
        endpointgroup = self.endpointgroups.first()
        networks = [{'subnets': [self.subnets.first(), ]}, ]

        self.mock_network_list_for_tenant.return_value = networks

        form_data = {'name': endpointgroup['name'],
                     'description': endpointgroup['description'],
                     'endpoints': endpointgroup['endpoints'],
                     'type': ''}

        res = self.client.post(reverse(self.ADDENDPOINTGROUP_PATH), form_data)

        self.assertFormErrors(res, 1)
        self.mock_network_list_for_tenant.assert_called_once_with(
            helpers.IsHttpRequest(), self.tenant.id)

    def test_add_ikepolicy_get(self):
        res = self.client.get(reverse(self.ADDIKEPOLICY_PATH))

        workflow = res.context['workflow']
        self.assertTemplateUsed(res, views.WorkflowView.template_name)
        self.assertEqual(workflow.name, workflows.AddIKEPolicy.name)

        expected_objs = ['<AddIKEPolicyStep: addikepolicyaction>', ]
        steps = [repr(step) for step in workflow.steps]
        self.assertQuerySetEqual(steps, expected_objs)

    @helpers.create_mocks({api_vpn: ('ikepolicy_create', )})
    def test_add_ikepolicy_post(self):
        ikepolicy = self.ikepolicies.first()

        form_data = {'name': ikepolicy['name'],
                     'description': ikepolicy['description'],
                     'auth_algorithm': ikepolicy['auth_algorithm'],
                     'encryption_algorithm': ikepolicy[
                         'encryption_algorithm'],
                     'ike_version': ikepolicy['ike_version'],
                     'lifetime_units': ikepolicy['lifetime']['units'],
                     'lifetime_value': ikepolicy['lifetime']['value'],
                     'phase1_negotiation_mode': ikepolicy[
                         'phase1_negotiation_mode'],
                     'pfs': ikepolicy['pfs']}

        self.mock_ikepolicy_create.return_value = ikepolicy

        res = self.client.post(reverse(self.ADDIKEPOLICY_PATH), form_data)

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, str(self.INDEX_URL))
        data = form_data.copy()
        data['lifetime'] = {'units': data['lifetime_units'],
                            'value': data['lifetime_value']}
        del data['lifetime_units']
        del data['lifetime_value']
        self.mock_ikepolicy_create.assert_called_once_with(
            helpers.IsHttpRequest(), **data)

    def test_add_ikepolicy_post_error(self):
        ikepolicy = self.ikepolicies.first()

        form_data = {'name': ikepolicy['name'],
                     'description': ikepolicy['description'],
                     'auth_algorithm': ikepolicy['auth_algorithm'],
                     'encryption_algorithm': ikepolicy[
                         'encryption_algorithm'],
                     'ike_version': ikepolicy['ike_version'],
                     'lifetime_units': ikepolicy['lifetime']['units'],
                     'lifetime_value': 10,
                     'phase1_negotiation_mode': ikepolicy[
                         'phase1_negotiation_mode'],
                     'pfs': ikepolicy['pfs']}

        res = self.client.post(reverse(self.ADDIKEPOLICY_PATH), form_data)

        self.assertFormErrors(res, 1)

    def test_add_ipsecpolicy_get(self):
        res = self.client.get(reverse(self.ADDIPSECPOLICY_PATH))

        workflow = res.context['workflow']
        self.assertTemplateUsed(res, views.WorkflowView.template_name)
        self.assertEqual(workflow.name, workflows.AddIPsecPolicy.name)

        expected_objs = ['<AddIPsecPolicyStep: addipsecpolicyaction>', ]
        steps = [repr(step) for step in workflow.steps]
        self.assertQuerySetEqual(steps, expected_objs)

    @helpers.create_mocks({api_vpn: ('ipsecpolicy_create', )})
    def test_add_ipsecpolicy_post(self):
        ipsecpolicy = self.ipsecpolicies.first()

        form_data = {'name': ipsecpolicy['name'],
                     'description': ipsecpolicy['description'],
                     'auth_algorithm': ipsecpolicy['auth_algorithm'],
                     'encryption_algorithm': ipsecpolicy[
                         'encryption_algorithm'],
                     'encapsulation_mode': ipsecpolicy[
                         'encapsulation_mode'],
                     'lifetime_units': ipsecpolicy['lifetime']['units'],
                     'lifetime_value': ipsecpolicy['lifetime']['value'],
                     'pfs': ipsecpolicy['pfs'],
                     'transform_protocol': ipsecpolicy[
                         'transform_protocol']}

        self.mock_ipsecpolicy_create.return_value = ipsecpolicy

        res = self.client.post(reverse(self.ADDIPSECPOLICY_PATH), form_data)

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, str(self.INDEX_URL))

        data = form_data.copy()
        data['lifetime'] = {'units': data['lifetime_units'],
                            'value': data['lifetime_value']}
        del data['lifetime_units']
        del data['lifetime_value']
        self.mock_ipsecpolicy_create.assert_called_once_with(
            helpers.IsHttpRequest(), **data)

    def test_add_ipsecpolicy_post_error(self):
        ipsecpolicy = self.ipsecpolicies.first()

        form_data = {'name': ipsecpolicy['name'],
                     'description': ipsecpolicy['description'],
                     'auth_algorithm': ipsecpolicy['auth_algorithm'],
                     'encryption_algorithm': ipsecpolicy[
                         'encryption_algorithm'],
                     'encapsulation_mode': ipsecpolicy[
                         'encapsulation_mode'],
                     'lifetime_units': ipsecpolicy['lifetime']['units'],
                     'lifetime_value': 10,
                     'pfs': ipsecpolicy['pfs'],
                     'transform_protocol': ipsecpolicy[
                         'transform_protocol']}

        res = self.client.post(reverse(self.ADDIPSECPOLICY_PATH), form_data)

        self.assertFormErrors(res, 1)

    @helpers.create_mocks({api_vpn: ('ikepolicy_list',
                                     'ipsecpolicy_list',
                                     'vpnservice_list',
                                     'endpointgroup_list',)})
    def test_add_ipsecsiteconnection_get(self):
        ikepolicies = self.ikepolicies.list()
        ipsecpolicies = self.ipsecpolicies.list()
        vpnservices = self.vpnservices.list()

        self.mock_ikepolicy_list.return_value = ikepolicies
        self.mock_ipsecpolicy_list.return_value = ipsecpolicies
        self.mock_vpnservice_list.return_value = vpnservices

        res = self.client.get(reverse(self.ADDVPNCONNECTION_PATH))

        workflow = res.context['workflow']
        self.assertTemplateUsed(res, views.WorkflowView.template_name)
        self.assertEqual(workflow.name, workflows.AddIPsecSiteConnection.name)

        expected_objs = ['<AddIPsecSiteConnectionStep: '
                         'addipsecsiteconnectionaction>',
                         '<AddIPsecSiteConnectionOptionalStep: '
                         'addipsecsiteconnectionoptionalaction>', ]
        steps = [repr(step) for step in workflow.steps]
        self.assertQuerySetEqual(steps, expected_objs)

        self.mock_ikepolicy_list.assert_called_once_with(
            helpers.IsHttpRequest(),
            tenant_id=self.tenant.id)
        self.mock_ipsecpolicy_list.assert_called_once_with(
            helpers.IsHttpRequest(),
            tenant_id=self.tenant.id)
        self.mock_vpnservice_list.assert_called_once_with(
            helpers.IsHttpRequest(),
            tenant_id=self.tenant.id)

    @helpers.create_mocks({api_vpn: ('ikepolicy_list', 'ipsecpolicy_list',
                                     'vpnservice_list', 'endpointgroup_list',
                                     'ipsecsiteconnection_create')})
    def test_add_ipsecsiteconnection_post(self):
        self._test_add_ipsecsiteconnection_post()

    @helpers.create_mocks({api_vpn: ('ikepolicy_list', 'ipsecpolicy_list',
                                     'vpnservice_list', 'endpointgroup_list',
                                     'ipsecsiteconnection_create')})
    def test_add_ipsecsiteconnection_post_single_subnet(self):
        self._test_add_ipsecsiteconnection_post(subnet_list=False)

    def _test_add_ipsecsiteconnection_post(self, subnet_list=True):
        if subnet_list:
            ipsecsiteconnection = self.ipsecsiteconnections.first()
        else:
            ipsecsiteconnection = self.ipsecsiteconnections.list()[1]
        ikepolicies = self.ikepolicies.list()
        ipsecpolicies = self.ipsecpolicies.list()
        vpnservices = self.vpnservices.list()

        form_data = {
            'name': ipsecsiteconnection['name'],
            'description': ipsecsiteconnection['description'],
            'dpd_action': ipsecsiteconnection['dpd']['action'],
            'dpd_interval': ipsecsiteconnection['dpd']['interval'],
            'dpd_timeout': ipsecsiteconnection['dpd']['timeout'],
            'ikepolicy_id': ipsecsiteconnection['ikepolicy_id'],
            'initiator': ipsecsiteconnection['initiator'],
            'ipsecpolicy_id': ipsecsiteconnection['ipsecpolicy_id'],
            'mtu': ipsecsiteconnection['mtu'],
            'peer_address': ipsecsiteconnection['peer_address'],
            'peer_cidrs': ', '.join(ipsecsiteconnection['peer_cidrs']),
            'peer_id': ipsecsiteconnection['peer_id'],
            'psk': ipsecsiteconnection['psk'],
            'vpnservice_id': ipsecsiteconnection['vpnservice_id'],
            'admin_state_up': ipsecsiteconnection['admin_state_up'],
        }

        self.mock_ikepolicy_list.return_value = ikepolicies
        self.mock_ipsecpolicy_list.return_value = ipsecpolicies
        self.mock_vpnservice_list.return_value = vpnservices
        self.mock_ipsecsiteconnection_create.return_value = ipsecsiteconnection

        res = self.client.post(reverse(self.ADDVPNCONNECTION_PATH), form_data)

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, str(self.INDEX_URL))

        self.mock_ikepolicy_list.assert_called_once_with(
            helpers.IsHttpRequest(),
            tenant_id=self.tenant.id)
        self.mock_ipsecpolicy_list.assert_called_once_with(
            helpers.IsHttpRequest(),
            tenant_id=self.tenant.id)
        self.mock_vpnservice_list.assert_called_once_with(
            helpers.IsHttpRequest(),
            tenant_id=self.tenant.id)

        data = form_data.copy()
        data['peer_cidrs'] = ipsecsiteconnection['peer_cidrs']
        data['dpd'] = {'action': data['dpd_action'],
                       'interval': data['dpd_interval'],
                       'timeout': data['dpd_timeout']}
        del data['dpd_action']
        del data['dpd_interval']
        del data['dpd_timeout']
        data['local_ep_group_id'] = ''
        data['peer_ep_group_id'] = ''

        self.mock_ipsecsiteconnection_create.assert_called_once_with(
            helpers.IsHttpRequest(), **data)

    @helpers.create_mocks({api_vpn: ('ikepolicy_list', 'ipsecpolicy_list',
                                     'vpnservice_list', 'endpointgroup_list',
                                     'ipsecsiteconnection_create')})
    def test_add_ipsecsiteconnection_post_required_fields_error(self):
        self._test_add_ipsecsiteconnection_post_error()

    @helpers.create_mocks({api_vpn: ('ikepolicy_list', 'ipsecpolicy_list',
                                     'vpnservice_list', 'endpointgroup_list',
                                     'ipsecsiteconnection_create')})
    def test_add_ipsecsiteconnection_post_peer_cidrs_error(self):
        self._test_add_ipsecsiteconnection_post_error(subnets=True)

    def _test_add_ipsecsiteconnection_post_error(self, subnets=False):
        ipsecsiteconnection = self.ipsecsiteconnections.first()
        ikepolicies = self.ikepolicies.list()
        ipsecpolicies = self.ipsecpolicies.list()
        vpnservices = self.vpnservices.list()

        self.mock_ikepolicy_list.return_value = ikepolicies
        self.mock_ipsecpolicy_list.return_value = ipsecpolicies
        self.mock_vpnservice_list.return_value = vpnservices

        form_data = {'name': '',
                     'description': ipsecsiteconnection['description'],
                     'dpd_action': ipsecsiteconnection['dpd']['action'],
                     'dpd_interval': ipsecsiteconnection['dpd']['interval'],
                     'dpd_timeout': ipsecsiteconnection['dpd']['timeout'],
                     'ikepolicy_id': '',
                     'initiator': ipsecsiteconnection['initiator'],
                     'ipsecpolicy_id': '',
                     'mtu': ipsecsiteconnection['mtu'],
                     'peer_address': '',
                     'peer_cidrs': '',
                     'peer_id': '',
                     'psk': '',
                     'vpnservice_id': '',
                     'admin_state_up': ipsecsiteconnection[
                         'admin_state_up']}
        if subnets:
            form_data['peer_cidrs'] = '20.1.0.0/24; 21.1.0.0/24'

        res = self.client.post(reverse(self.ADDVPNCONNECTION_PATH), form_data)

        if subnets:
            self.assertFormErrors(res, 7)
        else:
            self.assertFormErrors(res, 6)

        self.mock_ikepolicy_list.assert_called_once_with(
            helpers.IsHttpRequest(),
            tenant_id=self.tenant.id)
        self.mock_ipsecpolicy_list.assert_called_once_with(
            helpers.IsHttpRequest(),
            tenant_id=self.tenant.id)
        self.mock_vpnservice_list.assert_called_once_with(
            helpers.IsHttpRequest(),
            tenant_id=self.tenant.id)

    @helpers.create_mocks({api_vpn: ('vpnservice_get', )})
    def test_update_vpnservice_get(self):
        vpnservice = self.vpnservices.first()

        self.mock_vpnservice_get.return_value = vpnservice

        res = self.client.get(
            reverse(self.UPDATEVPNSERVICE_PATH, args=(vpnservice.id,)))

        self.assertTemplateUsed(
            res, 'project/vpn/update_vpnservice.html')

        self.mock_vpnservice_get.assert_called_once_with(
            helpers.IsHttpRequest(), vpnservice.id)

    @helpers.create_mocks({api_vpn: ('vpnservice_get', 'vpnservice_update')})
    def test_update_vpnservice_post(self):
        vpnservice = self.vpnservices.first()

        self.mock_vpnservice_get.return_value = vpnservice

        data = {'name': vpnservice.name,
                'description': vpnservice.description,
                'admin_state_up': vpnservice.admin_state_up}

        self.mock_vpnservice_update.return_value = vpnservice

        form_data = data.copy()
        form_data['vpnservice_id'] = vpnservice.id

        res = self.client.post(reverse(
            self.UPDATEVPNSERVICE_PATH, args=(vpnservice.id,)), form_data)

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, str(self.INDEX_URL))

        self.mock_vpnservice_get.assert_called_once_with(
            helpers.IsHttpRequest(), vpnservice.id)
        self.mock_vpnservice_update.assert_called_once_with(
            helpers.IsHttpRequest(), vpnservice.id, vpnservice=data)

    @helpers.create_mocks({api_vpn: ('endpointgroup_get', )})
    def test_update_endpointgroup_get(self):
        endpointgroup = self.endpointgroups.first()

        self.mock_endpointgroup_get.return_value = endpointgroup

        res = self.client.get(
            reverse(self.UPDATEENDPOINTGROUP_PATH, args=(endpointgroup.id,)))

        self.assertTemplateUsed(
            res, 'project/vpn/update_endpointgroup.html')

        self.mock_endpointgroup_get.assert_called_once_with(
            helpers.IsHttpRequest(), endpointgroup.id)

    @helpers.create_mocks({api_vpn: ('endpointgroup_get',
                                     'endpointgroup_update')})
    def test_update_endpointgroup_post(self):
        endpointgroup = self.endpointgroups.first()

        data = {'name': endpointgroup.name,
                'description': endpointgroup.description}

        self.mock_endpointgroup_get.return_value = endpointgroup
        self.mock_endpointgroup_update.return_value = endpointgroup

        form_data = data.copy()
        form_data['endpoint_group_id'] = endpointgroup.id

        res = self.client.post(reverse(self.UPDATEENDPOINTGROUP_PATH,
                                       args=(endpointgroup.id, )
                                       ), form_data)

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, str(self.INDEX_URL))

        self.mock_endpointgroup_get.assert_called_once_with(
            helpers.IsHttpRequest(), endpointgroup.id)
        self.mock_endpointgroup_update.assert_called_once_with(
            helpers.IsHttpRequest(), endpointgroup.id, endpoint_group=data)

    @helpers.create_mocks({api_vpn: ('ikepolicy_get', )})
    def test_update_ikepolicy_get(self):
        ikepolicy = self.ikepolicies.first()

        self.mock_ikepolicy_get.return_value = ikepolicy

        res = self.client.get(
            reverse(self.UPDATEIKEPOLICY_PATH, args=(ikepolicy.id,)))

        self.assertTemplateUsed(
            res, 'project/vpn/update_ikepolicy.html')
        self.mock_ikepolicy_get.assert_called_once_with(
            helpers.IsHttpRequest(), ikepolicy.id)

    @helpers.create_mocks({api_vpn: ('ikepolicy_get', 'ikepolicy_update')})
    def test_update_ikepolicy_post(self):
        ikepolicy = self.ikepolicies.first()

        data = {'name': ikepolicy.name,
                'description': ikepolicy.description,
                'auth_algorithm': ikepolicy.auth_algorithm,
                'encryption_algorithm': ikepolicy.encryption_algorithm,
                'ike_version': ikepolicy.ike_version,
                'lifetime': ikepolicy.lifetime,
                'pfs': ikepolicy.pfs,
                'phase1_negotiation_mode': ikepolicy.phase1_negotiation_mode}

        self.mock_ikepolicy_get.return_value = ikepolicy
        self.mock_ikepolicy_update.return_value = ikepolicy

        form_data = data.copy()

        form_data.update({'lifetime_units': form_data['lifetime']['units'],
                          'lifetime_value': form_data['lifetime']['value'],
                          'ikepolicy_id': ikepolicy.id})
        form_data.pop('lifetime')

        res = self.client.post(reverse(
            self.UPDATEIKEPOLICY_PATH, args=(ikepolicy.id,)), form_data)

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, str(self.INDEX_URL))

        self.mock_ikepolicy_get.assert_called_once_with(
            helpers.IsHttpRequest(), ikepolicy.id)
        self.mock_ikepolicy_update.assert_called_once_with(
            helpers.IsHttpRequest(), ikepolicy.id, ikepolicy=data)

    @helpers.create_mocks({api_vpn: ('ipsecpolicy_get', )})
    def test_update_ipsecpolicy_get(self):
        ipsecpolicy = self.ipsecpolicies.first()

        self.mock_ipsecpolicy_get.return_value = ipsecpolicy

        res = self.client.get(
            reverse(self.UPDATEIPSECPOLICY_PATH, args=(ipsecpolicy.id,)))

        self.assertTemplateUsed(
            res, 'project/vpn/update_ipsecpolicy.html')

        self.mock_ipsecpolicy_get.assert_called_once_with(
            helpers.IsHttpRequest(), ipsecpolicy.id)

    @helpers.create_mocks({api_vpn: ('ipsecpolicy_get', 'ipsecpolicy_update')})
    def test_update_ipsecpolicy_post(self):
        ipsecpolicy = self.ipsecpolicies.first()

        self.mock_ipsecpolicy_get.return_value = ipsecpolicy

        data = {'name': ipsecpolicy.name,
                'description': ipsecpolicy.description,
                'auth_algorithm': ipsecpolicy.auth_algorithm,
                'encapsulation_mode': ipsecpolicy.encapsulation_mode,
                'encryption_algorithm': ipsecpolicy.encryption_algorithm,
                'lifetime': ipsecpolicy.lifetime,
                'pfs': ipsecpolicy.pfs,
                'transform_protocol': ipsecpolicy.transform_protocol}

        self.mock_ipsecpolicy_update.return_value = ipsecpolicy

        form_data = data.copy()

        form_data.update({'lifetime_units': form_data['lifetime']['units'],
                          'lifetime_value': form_data['lifetime']['value'],
                          'ipsecpolicy_id': ipsecpolicy.id})
        form_data.pop('lifetime')

        res = self.client.post(reverse(
            self.UPDATEIPSECPOLICY_PATH, args=(ipsecpolicy.id,)), form_data)

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, str(self.INDEX_URL))

        self.mock_ipsecpolicy_get.assert_called_once_with(
            helpers.IsHttpRequest(), ipsecpolicy.id)
        self.mock_ipsecpolicy_update.assert_called_once_with(
            helpers.IsHttpRequest(), ipsecpolicy.id, ipsecpolicy=data)

    @helpers.create_mocks({api_vpn: ('ipsecsiteconnection_get', )})
    def test_update_ipsecsiteconnection_get(self):
        ipsecsiteconnection = self.ipsecsiteconnections.first()

        self.mock_ipsecsiteconnection_get.return_value = ipsecsiteconnection

        res = self.client.get(
            reverse(self.UPDATEVPNCONNECTION_PATH,
                    args=(ipsecsiteconnection.id,)))

        self.assertTemplateUsed(
            res, 'project/vpn/update_ipsecsiteconnection.html')
        self.mock_ipsecsiteconnection_get.assert_called_once_with(
            helpers.IsHttpRequest(), ipsecsiteconnection.id)

    @helpers.create_mocks({api_vpn: ('ipsecsiteconnection_get',
                                     'ipsecsiteconnection_update')})
    def test_update_ipsecsiteconnection_post(self):
        ipsecsiteconnection = self.ipsecsiteconnections.first()

        self.mock_ipsecsiteconnection_get.return_value = ipsecsiteconnection

        data = {'name': ipsecsiteconnection.name,
                'description': ipsecsiteconnection.description,
                'peer_address': ipsecsiteconnection.peer_address,
                'peer_id': ipsecsiteconnection.peer_id,
                'peer_cidrs': ipsecsiteconnection.peer_cidrs,
                'psk': ipsecsiteconnection.psk,
                'mtu': ipsecsiteconnection.mtu,
                'dpd': ipsecsiteconnection.dpd,
                'initiator': ipsecsiteconnection.initiator,
                'admin_state_up': ipsecsiteconnection.admin_state_up}

        self.mock_ipsecsiteconnection_update.return_value = ipsecsiteconnection

        form_data = data.copy()

        form_data.update({
            'dpd_action': form_data['dpd']['action'],
            'dpd_interval': form_data['dpd']['interval'],
            'dpd_timeout': form_data['dpd']['timeout'],
            'peer_cidrs': ", ".join(ipsecsiteconnection['peer_cidrs']),
            'ipsecsiteconnection_id': ipsecsiteconnection.id,
        })
        form_data.pop('dpd')

        res = self.client.post(
            reverse(self.UPDATEVPNCONNECTION_PATH,
                    args=(ipsecsiteconnection.id,)), form_data)

        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, str(self.INDEX_URL))

        self.mock_ipsecsiteconnection_get.assert_called_once_with(
            helpers.IsHttpRequest(), ipsecsiteconnection.id)
        self.mock_ipsecsiteconnection_update.assert_called_once_with(
            helpers.IsHttpRequest(), ipsecsiteconnection.id,
            ipsec_site_connection=data)

    @helpers.create_mocks({api_vpn: ('vpnservice_list', 'vpnservice_delete',)})
    def test_delete_vpnservice(self):
        vpnservice = self.vpnservices.list()[1]
        self.mock_vpnservice_list.return_value = self.vpnservices.list()
        self.mock_vpnservice_delete.return_value = None

        form_data = {"action":
                     "vpnservicestable__deletevpnservice__%s" % vpnservice.id}
        res = self.client.post(self.INDEX_URL, form_data)

        self.assertNoFormErrors(res)

        self.mock_vpnservice_list.assert_called_once_with(
            helpers.IsHttpRequest(), tenant_id=self.tenant.id)
        self.mock_vpnservice_delete.assert_called_once_with(
            helpers.IsHttpRequest(), vpnservice.id)

    @helpers.create_mocks({api_vpn: ('endpointgroup_list',
                                     'endpointgroup_delete',)})
    def test_delete_endpointgroup(self):
        endpointgroup = self.endpointgroups.list()[0]
        self.mock_endpointgroup_list.return_value = self.endpointgroups.list()
        self.mock_endpointgroup_delete.return_value = None

        form_data = {"action":
                     "endpointgroupstable__deleteendpointgroup__%s"
                     % endpointgroup.id}
        res = self.client.post(self.INDEX_URL, form_data)

        self.assertNoFormErrors(res)

        self.mock_endpointgroup_list.assert_called_once_with(
            helpers.IsHttpRequest(), tenant_id=self.tenant.id)
        self.mock_endpointgroup_delete.assert_called_once_with(
            helpers.IsHttpRequest(), endpointgroup.id)

    @helpers.create_mocks({api_vpn: ('ikepolicy_list', 'ikepolicy_delete',)})
    def test_delete_ikepolicy(self):
        ikepolicy = self.ikepolicies.list()[1]
        self.mock_ikepolicy_list.return_value = self.ikepolicies.list()
        self.mock_ikepolicy_delete.return_value = None

        form_data = {"action":
                     "ikepoliciestable__deleteikepolicy__%s" % ikepolicy.id}
        res = self.client.post(self.INDEX_URL, form_data)

        self.assertNoFormErrors(res)

        self.mock_ikepolicy_list.assert_called_once_with(
            helpers.IsHttpRequest(), tenant_id=self.tenant.id)
        self.mock_ikepolicy_delete.assert_called_once_with(
            helpers.IsHttpRequest(), ikepolicy.id)

    @helpers.create_mocks({api_vpn: ('ipsecpolicy_list',
                                     'ipsecpolicy_delete',)})
    def test_delete_ipsecpolicy(self):
        ipsecpolicy = self.ipsecpolicies.list()[1]
        self.mock_ipsecpolicy_list.return_value = self.ipsecpolicies.list()
        self.mock_ipsecpolicy_delete.return_value = None

        form_data = {"action":
                     "ipsecpoliciestable__deleteipsecpolicy__%s"
                     % ipsecpolicy.id}
        res = self.client.post(self.INDEX_URL, form_data)

        self.assertNoFormErrors(res)

        self.mock_ipsecpolicy_list.assert_called_once_with(
            helpers.IsHttpRequest(), tenant_id=self.tenant.id)
        self.mock_ipsecpolicy_delete.assert_called_once_with(
            helpers.IsHttpRequest(), ipsecpolicy.id)

    @helpers.create_mocks({api_vpn: ('ipsecsiteconnection_list',
                                     'ipsecsiteconnection_delete',)})
    def test_delete_ipsecsiteconnection(self):
        ipsecsiteconnection = self.ipsecsiteconnections.first()
        self.mock_ipsecsiteconnection_list.return_value = \
            self.ipsecsiteconnections.list()
        self.mock_ipsecsiteconnection_delete.return_value = None

        form_data = {"action":
                     "ipsecsiteconnectionstable__deleteipsecsiteconnection__%s"
                     % ipsecsiteconnection.id}
        res = self.client.post(self.INDEX_URL, form_data)

        self.assertNoFormErrors(res)

        self.mock_ipsecsiteconnection_list.assert_called_once_with(
            helpers.IsHttpRequest(), tenant_id=self.tenant.id)
        self.mock_ipsecsiteconnection_delete.assert_called_once_with(
            helpers.IsHttpRequest(), ipsecsiteconnection.id)



# Copyright 2013, Mirantis Inc
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import re_path

from neutron_vpnaas_dashboard.dashboards.project.vpn import views

urlpatterns = [
    re_path(r'^$', views.IndexView.as_view(), name='index'),
    re_path(r'^addikepolicy$',
            views.AddIKEPolicyView.as_view(), name='addikepolicy'),
    re_path(r'^update_ikepolicy/(?P<ikepolicy_id>[^/]+)/$',
            views.UpdateIKEPolicyView.as_view(), name='update_ikepolicy'),
    re_path(r'^addipsecpolicy$',
            views.AddIPsecPolicyView.as_view(), name='addipsecpolicy'),
    re_path(r'^update_ipsecpolicy/(?P<ipsecpolicy_id>[^/]+)/$',
            views.UpdateIPsecPolicyView.as_view(), name='update_ipsecpolicy'),
    re_path(r'^addipsecsiteconnection$',
            views.AddIPsecSiteConnectionView.as_view(),
            name='addipsecsiteconnection'),
    re_path(r'^update_ipsecsiteconnection/(?P<ipsecsiteconnection_id>[^/]+)/$',
            views.UpdateIPsecSiteConnectionView.as_view(),
            name='update_ipsecsiteconnection'),
    re_path(r'^addvpnservice$',
            views.AddVPNServiceView.as_view(), name='addvpnservice'),
    re_path(r'^update_vpnservice/(?P<vpnservice_id>[^/]+)/$',
            views.UpdateVPNServiceView.as_view(), name='update_vpnservice'),
    re_path(r'^addendpointgroup$',
            views.AddEndpointGroupView.as_view(), name='addendpointgroup'),
    re_path(r'^update_endpointgroup/(?P<endpoint_group_id>[^/]+)/$',
            views.UpdateEndpointGroupView.as_view(),
            name='update_endpointgroup'),
    re_path(r'^ikepolicy/(?P<ikepolicy_id>[^/]+)/$',
            views.IKEPolicyDetailsView.as_view(), name='ikepolicydetails'),
    re_path(r'^ipsecpolicy/(?P<ipsecpolicy_id>[^/]+)/$',
            views.IPsecPolicyDetailsView.as_view(), name='ipsecpolicydetails'),
    re_path(r'^vpnservice/(?P<vpnservice_id>[^/]+)/$',
            views.VPNServiceDetailsView.as_view(), name='vpnservicedetails'),
    re_path(r'^endpointgroup/(?P<endpoint_group_id>[^/]+)/$',
            views.EndpointGroupDetailsView.as_view(),
            name='endpointgroupdetails'),
    re_path(r'^ipsecsiteconnection/(?P<ipsecsiteconnection_id>[^/]+)/$',
            views.IPsecSiteConnectionDetailsView.as_view(),
            name='ipsecsiteconnectiondetails'),
]



# Copyright 2013, Mirantis Inc
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import messages

from neutron_vpnaas_dashboard.api import vpn as api_vpn


LOG = logging.getLogger(__name__)


class UpdateVPNService(forms.SelfHandlingForm):
    name = forms.CharField(max_length=80, label=_("Name"), required=False)
    description = forms.CharField(
        required=False, max_length=80, label=_("Description"))
    admin_state_up = forms.BooleanField(label=_("Enable Admin State"),
                                        required=False)

    failure_url = 'horizon:project:vpn:index'

    def handle(self, request, context):
        try:
            data = {'vpnservice': {'name': context['name'],
                                   'description': context['description'],
                                   'admin_state_up': context['admin_state_up'],
                                   }}
            vpnservice = api_vpn.vpnservice_update(
                request, self.initial['vpnservice_id'], **data)
            msg = (_('VPN service %s was successfully updated.')
                   % context['name'])
            messages.success(request, msg)
            return vpnservice
        except Exception as e:
            LOG.info('Failed to update VPN service %(id)s: %(exc)s',
                     {'id': self.initial['vpnservice_id'], 'exc': e})
            msg = _('Failed to update VPN service %s') % context['name']
            redirect = reverse(self.failure_url)
            exceptions.handle(request, msg, redirect=redirect)


class UpdateEndpointGroup(forms.SelfHandlingForm):
    name = forms.CharField(
        max_length=80,
        label=_("Name"),
        required=False)
    description = forms.CharField(
        required=False,
        max_length=80,
        label=_("Description"))

    failure_url = 'horizon:project:vpn:index'

    def handle(self, request, context):
        try:
            data = {'endpoint_group':
                    {'name': context['name'],
                     'description': context['description']}
                    }
            endpointgroup = api_vpn.endpointgroup_update(
                request, self.initial['endpoint_group_id'], **data)
            msg = (_('Endpoint group %s was successfully updated.')
                   % context['name'])
            messages.success(request, msg)
            return endpointgroup
        except Exception as e:
            LOG.info('Failed to update endpoint group %(id)s: %(exc)s',
                     {'id': self.initial['endpoint_group_id'], 'exc': e})
            msg = _('Failed to update endpoint group %s') % context['name']
            redirect = reverse(self.failure_url)
            exceptions.handle(request, msg, redirect=redirect)


class UpdateIKEPolicy(forms.SelfHandlingForm):
    name = forms.CharField(max_length=80, label=_("Name"), required=False)
    description = forms.CharField(
        required=False, max_length=80, label=_("Description"))
    auth_algorithm = forms.ThemableChoiceField(
        label=_("Authorization algorithm"),
        choices=[('sha1', _('SHA-1')),
                 ('sha256', _('SHA-256')),
                 ('sha384', _('SHA-384')),
                 ('sha512', _('SHA-512')),
                 ('aes-xcbc', _('AES-XCBC')),
                 ('aes-cmac', _('AES-CMAC'))],
        required=False)
    encryption_algorithm = forms.ThemableChoiceField(
        label=_("Encryption algorithm"),
        choices=[('3des', _('3DES')),
                 ('aes-128', _('AES-128')),
                 ('aes-192', _('AES-192')),
                 ('aes-256', _('AES-256')),
                 ('aes-128-ctr', _('AES-128-CTR')),
                 ('aes-192-ctr', _('AES-192-CTR')),
                 ('aes-256-ctr', _('AES-256-CTR')),
                 ('aes-128-ccm-8', _('AES-128-CCM-8')),
                 ('aes-192-ccm-8', _('AES-192-CCM-8')),
                 ('aes-256-ccm-8', _('AES-256-CCM-8')),
                 ('aes-128-ccm-12', _('AES-128-CCM-12')),
                 ('aes-192-ccm-12', _('AES-192-CCM-12')),
                 ('aes-256-ccm-12', _('AES-256-CCM-12')),
                 ('aes-128-ccm-16', _('AES-128-CCM-16')),
                 ('aes-192-ccm-16', _('AES-192-CCM-16')),
                 ('aes-256-ccm-16', _('AES-256-CCM-16')),
                 ('aes-128-gcm-8', _('AES-128-GCM-8')),
                 ('aes-192-gcm-8', _('AES-192-GCM-8')),
                 ('aes-256-gcm-8', _('AES-256-GCM-8')),
                 ('aes-128-gcm-12', _('AES-128-GCM-12')),
                 ('aes-192-gcm-12', _('AES-192-GCM-12')),
                 ('aes-256-gcm-12', _('AES-256-GCM-12')),
                 ('aes-128-gcm-16', _('AES-128-GCM-16')),
                 ('aes-192-gcm-16', _('AES-192-GCM-16')),
                 ('aes-256-gcm-16', _('AES-256-GCM-16'))],
        required=False)
    ike_version = forms.ThemableChoiceField(
        label=_("IKE version"),
        choices=[('v1', _('v1')),
                 ('v2', _('v2'))],
        required=False)
    # Currently this field has only one choice, so mark it as readonly.
    lifetime_units = forms.ThemableChoiceField(
        label=_("Lifetime units for IKE keys"),
        choices=[('seconds', _('seconds'))],
        widget=forms.TextInput(attrs={'readonly': 'readonly'}),
        required=False)
    lifetime_value = forms.IntegerField(
        min_value=60,
        label=_("Lifetime value for IKE keys"),
        help_text=_("Equal to or greater than 60"),
        required=False)
    pfs = forms.ThemableChoiceField(
        label=_("Perfect Forward Secrecy"),
        choices=[('group2', _('group2')),
                 ('group5', _('group5')),
                 ('group14', _('group14')),
                 ('group15', _('group15')),
                 ('group16', _('group16')),
                 ('group17', _('group17')),
                 ('group18', _('group18')),
                 ('group19', _('group19')),
                 ('group20', _('group20')),
                 ('group21', _('group21')),
                 ('group22', _('group22')),
                 ('group23', _('group23')),
                 ('group24', _('group24')),
                 ('group25', _('group25')),
                 ('group26', _('group26')),
                 ('group27', _('group27')),
                 ('group28', _('group28')),
                 ('group29', _('group29')),
                 ('group30', _('group30')),
                 ('group31', _('group31'))],
        required=False)
    phase1_negotiation_mode = forms.ThemableChoiceField(
        label=_("IKE Phase1 negotiation mode"),
        choices=[('main', 'main'),
                 ('aggressive', 'aggressive')],
        required=False)

    failure_url = 'horizon:project:vpn:index'

    def handle(self, request, context):
        try:
            data = {'ikepolicy':
                    {'name': context['name'],
                     'description': context['description'],
                     'auth_algorithm': context['auth_algorithm'],
                     'encryption_algorithm': context['encryption_algorithm'],
                     'ike_version': context['ike_version'],
                     'lifetime': {'units': context['lifetime_units'],
                                  'value': context['lifetime_value']},
                     'pfs': context['pfs'],
                     'phase1_negotiation_mode':
                     context['phase1_negotiation_mode'],
                     }}
            ikepolicy = api_vpn.ikepolicy_update(
                request, self.initial['ikepolicy_id'], **data)
            msg = (_('IKE policy %s was successfully updated.')
                   % context['name'])
            messages.success(request, msg)
            return ikepolicy
        except Exception as e:
            LOG.info('Failed to update IKE policy %(id)s: %(exc)s',
                     {'id': self.initial['ikepolicy_id'], 'exc': e})
            msg = _('Failed to update IKE policy %s') % context['name']
            redirect = reverse(self.failure_url)
            exceptions.handle(request, msg, redirect=redirect)


class UpdateIPsecPolicy(forms.SelfHandlingForm):
    name = forms.CharField(max_length=80, label=_("Name"), required=False)
    description = forms.CharField(
        required=False, max_length=80, label=_("Description"))
    # Currently this field has only one choice, so mark it as readonly.
    auth_algorithm = forms.ThemableChoiceField(
        label=_("Authorization algorithm"),
        choices=[('sha1', _('SHA-1')),
                 ('sha256', _('SHA-256')),
                 ('sha384', _('SHA-384')),
                 ('sha512', _('SHA-512')),
                 ('aes-xcbc', _('AES-XCBC')),
                 ('aes-cmac', _('AES-CMAC'))],
        required=False)
    encapsulation_mode = forms.ThemableChoiceField(
        label=_("Encapsulation mode"),
        choices=[('tunnel', _('tunnel')),
                 ('transport', _('transport'))],
        required=False)
    encryption_algorithm = forms.ThemableChoiceField(
        label=_("Encryption algorithm"),
        choices=[('3des', _('3DES')),
                 ('aes-128', _('AES-128')),
                 ('aes-192', _('AES-192')),
                 ('aes-256', _('AES-256')),
                 ('aes-128-ctr', _('AES-128-CTR')),
                 ('aes-192-ctr', _('AES-192-CTR')),
                 ('aes-256-ctr', _('AES-256-CTR')),
                 ('aes-128-ccm-8', _('AES-128-CCM-8')),
                 ('aes-192-ccm-8', _('AES-192-CCM-8')),
                 ('aes-256-ccm-8', _('AES-256-CCM-8')),
                 ('aes-128-ccm-12', _('AES-128-CCM-12')),
                 ('aes-192-ccm-12', _('AES-192-CCM-12')),
                 ('aes-256-ccm-12', _('AES-256-CCM-12')),
                 ('aes-128-ccm-16', _('AES-128-CCM-16')),
                 ('aes-192-ccm-16', _('AES-192-CCM-16')),
                 ('aes-256-ccm-16', _('AES-256-CCM-16')),
                 ('aes-128-gcm-8', _('AES-128-GCM-8')),
                 ('aes-192-gcm-8', _('AES-192-GCM-8')),
                 ('aes-256-gcm-8', _('AES-256-GCM-8')),
                 ('aes-128-gcm-12', _('AES-128-GCM-12')),
                 ('aes-192-gcm-12', _('AES-192-GCM-12')),
                 ('aes-256-gcm-12', _('AES-256-GCM-12')),
                 ('aes-128-gcm-16', _('AES-128-GCM-16')),
                 ('aes-192-gcm-16', _('AES-192-GCM-16')),
                 ('aes-256-gcm-16', _('AES-256-GCM-16'))],
        required=False)
    # Currently this field has only one choice, so mark it as readonly.
    lifetime_units = forms.ThemableChoiceField(
        label=_("Lifetime units"),
        choices=[('seconds', _('seconds'))],
        widget=forms.TextInput(attrs={'readonly': 'readonly'}),
        required=False)
    lifetime_value = forms.IntegerField(
        min_value=60,
        label=_("Lifetime value"),
        help_text=_("Equal to or greater than 60"),
        required=False)
    pfs = forms.ThemableChoiceField(
        label=_("Perfect Forward Secrecy"),
        choices=[('group2', _('group2')),
                 ('group5', _('group5')),
                 ('group14', _('group14')),
                 ('group15', _('group15')),
                 ('group16', _('group16')),
                 ('group17', _('group17')),
                 ('group18', _('group18')),
                 ('group19', _('group19')),
                 ('group20', _('group20')),
                 ('group21', _('group21')),
                 ('group22', _('group22')),
                 ('group23', _('group23')),
                 ('group24', _('group24')),
                 ('group25', _('group25')),
                 ('group26', _('group26')),
                 ('group27', _('group27')),
                 ('group28', _('group28')),
                 ('group29', _('group29')),
                 ('group30', _('group30')),
                 ('group31', _('group31'))],
        required=False)
    transform_protocol = forms.ThemableChoiceField(
        label=_("Transform Protocol"),
        choices=[('esp', _('ESP')),
                 ('ah', _('AH')),
                 ('ah-esp', _('AH-ESP'))],
        required=False)

    failure_url = 'horizon:project:vpn:index'

    def handle(self, request, context):
        try:
            data = {'ipsecpolicy':
                    {'name': context['name'],
                     'description': context['description'],
                     'auth_algorithm': context['auth_algorithm'],
                     'encapsulation_mode': context['encapsulation_mode'],
                     'encryption_algorithm': context['encryption_algorithm'],
                     'lifetime': {'units': context['lifetime_units'],
                                  'value': context['lifetime_value']},
                     'pfs': context['pfs'],
                     'transform_protocol': context['transform_protocol'],
                     }}
            ipsecpolicy = api_vpn.ipsecpolicy_update(
                request, self.initial['ipsecpolicy_id'], **data)
            msg = (_('IPsec policy %s was successfully updated.')
                   % context['name'])
            messages.success(request, msg)
            return ipsecpolicy
        except Exception as e:
            LOG.info('Failed to update IPsec policy %(id)s: %(exc)s',
                     {'id': self.initial['ipsecpolicy_id'], 'exc': e})
            msg = _('Failed to update IPsec policy %s') % context['name']
            redirect = reverse(self.failure_url)
            exceptions.handle(request, msg, redirect=redirect)


class UpdateIPsecSiteConnection(forms.SelfHandlingForm):
    name = forms.CharField(max_length=80, label=_("Name"), required=False)
    description = forms.CharField(
        required=False, max_length=80, label=_("Description"))
    peer_address = forms.CharField(
        label=_("Peer gateway public IPv4/IPv6 Address or FQDN"),
        help_text=_("Peer gateway public IPv4/IPv6 address or FQDN for "
                    "the VPN Connection"),)
    peer_id = forms.CharField(
        label=_("Peer router identity for authentication (Peer ID)"),
        help_text=_("Peer router identity for authentication. "
                    "Can be IPv4/IPv6 address, e-mail, key ID, or FQDN"),)
    peer_cidrs = forms.MultiIPField(
        required=False,
        label=_("Remote peer subnet(s)"),
        help_text=_("Remote peer subnet(s) address(es) "
                    "with mask(s) in CIDR format "
                    "separated with commas if needed "
                    "(e.g. 20.1.0.0/24, 21.1.0.0/24)"),
        version=forms.IPv4 | forms.IPv6,
        mask=True)
    local_ep_group_id = forms.CharField(
        required=False,
        label=_("Local Endpoint Group(s)"),
        help_text=_("IPsec connection validation requires "
                    "that local endpoints are subnets"))
    peer_ep_group_id = forms.CharField(
        required=False,
        label=_("Peer Endpoint Group(s)"),
        help_text=_("IPsec connection validation requires "
                    "that peer endpoints are CIDRs"))
    psk = forms.CharField(
        widget=forms.PasswordInput(render_value=True),
        max_length=80, label=_("Pre-Shared Key (PSK) string"))
    mtu = forms.IntegerField(
        min_value=68,
        required=False,
        label=_("Maximum Transmission Unit size for the connection"),
        help_text=_("Equal to or greater than 68 if the local subnet is IPv4. "
                    "Equal to or greater than 1280 if the local subnet "
                    "is IPv6."))
    dpd_action = forms.ThemableChoiceField(
        label=_("Dead peer detection actions"),
        required=False,
        choices=[('hold', _('hold')),
                 ('clear', _('clear')),
                 ('disabled', _('disabled')),
                 ('restart', _('restart')),
                 ('restart-by-peer', _('restart-by-peer'))])
    dpd_interval = forms.IntegerField(
        min_value=1,
        required=False,
        label=_("Dead peer detection interval"),
        help_text=_("Valid integer lesser than the DPD timeout"))
    dpd_timeout = forms.IntegerField(
        min_value=1,
        required=False,
        label=_("Dead peer detection timeout"),
        help_text=_("Valid integer greater than the DPD interval"))
    initiator = forms.ThemableChoiceField(
        label=_("Initiator state"),
        required=False,
        choices=[('bi-directional', _('bi-directional')),
                 ('response-only', _('response-only'))])
    admin_state_up = forms.BooleanField(label=_("Enable Admin State"),
                                        required=False)

    failure_url = 'horizon:project:vpn:index'

    def clean(self):
        cleaned_data = super(UpdateIPsecSiteConnection, self).clean()
        interval = cleaned_data.get('dpd_interval')
        timeout = cleaned_data.get('dpd_timeout')

        if not interval < timeout:
            msg = _("DPD Timeout must be greater than DPD Interval")
            self._errors['dpd_timeout'] = self.error_class([msg])
        return cleaned_data

    def handle(self, request, context):
        try:
            data = {
                'name': context['name'],
                'description': context['description'],
                'peer_address': context['peer_address'],
                'peer_id': context['peer_id'],
                'psk': context['psk'],
                'mtu': context['mtu'],
                'dpd': {'action': context['dpd_action'],
                        'interval': context['dpd_interval'],
                        'timeout': context['dpd_timeout']},
                'initiator': context['initiator'],
                'admin_state_up': context['admin_state_up']
            }
            if not context['peer_cidrs']:
                data['local_ep_group_id'] = context['local_ep_group_id']
                data['peer_ep_group_id'] = context['peer_ep_group_id']
            else:
                cidrs = context['peer_cidrs']
                data['peer_cidrs'] = [cidr.strip() for cidr in cidrs.split(',')
                                      if cidr.strip()]
            ipsecsiteconnection = api_vpn.ipsecsiteconnection_update(
                request, self.initial['ipsecsiteconnection_id'],
                ipsec_site_connection=data)
            msg = (_('IPsec site connection %s was successfully updated.')
                   % context['name'])
            messages.success(request, msg)
            return ipsecsiteconnection
        except Exception as e:
            LOG.info('Failed to update IPsec site connection %(id)s: %(exc)s',
                     {'id': self.initial['ipsecsiteconnection_id'], 'exc': e})
            msg = (_('Failed to update IPsec site connection %s')
                   % context['name'])
            redirect = reverse(self.failure_url)
            exceptions.handle(request, msg, redirect=redirect)



# Copyright 2013, Mirantis Inc
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from django import template
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy
from django.utils.translation import pgettext_lazy
from horizon import exceptions
from horizon import tables
from openstack_dashboard import policy

from neutron_vpnaas_dashboard.api import vpn as api_vpn


forbid_updates = set(["PENDING_CREATE", "PENDING_UPDATE", "PENDING_DELETE"])


class AddIKEPolicyLink(tables.LinkAction):
    name = "addikepolicy"
    verbose_name = _("Add IKE Policy")
    url = "horizon:project:vpn:addikepolicy"
    classes = ("ajax-modal",)
    icon = "plus"
    policy_rules = (("network", "create_ikepolicy"),)


class AddIPsecPolicyLink(tables.LinkAction):
    name = "addipsecpolicy"
    verbose_name = _("Add IPsec Policy")
    url = "horizon:project:vpn:addipsecpolicy"
    classes = ("ajax-modal",)
    icon = "plus"
    policy_rules = (("network", "create_ipsecpolicy"),)


class AddVPNServiceLink(tables.LinkAction):
    name = "addvpnservice"
    verbose_name = _("Add VPN Service")
    url = "horizon:project:vpn:addvpnservice"
    classes = ("ajax-modal",)
    icon = "plus"
    policy_rules = (("network", "create_vpnservice"),)


class AddEndpointGroupLink(tables.LinkAction):
    name = "addendpointgroup"
    verbose_name = _("Add Endpoint Group")
    url = "horizon:project:vpn:addendpointgroup"
    classes = ("ajax-modal",)
    icon = "plus"
    policy_rules = (("network", "create_endpointgroup"),)


class AddIPsecSiteConnectionLink(tables.LinkAction):
    name = "addipsecsiteconnection"
    verbose_name = _("Add IPsec Site Connection")
    url = "horizon:project:vpn:addipsecsiteconnection"
    classes = ("ajax-modal",)
    icon = "plus"
    policy_rules = (("network", "create_ipsec_site_connection"),)


class DeleteVPNServiceLink(policy.PolicyTargetMixin, tables.DeleteAction):
    name = "deletevpnservice"
    policy_rules = (("network", "delete_vpnservice"),)

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Delete VPN Service",
            "Delete VPN Services",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Scheduled deletion of VPN service",
            "Scheduled deletion of VPN services",
            count
        )

    def allowed(self, request, datum=None):
        if datum and datum.ipsecsiteconns:
            return False
        return True

    def delete(self, request, obj_id):
        try:
            api_vpn.vpnservice_delete(request, obj_id)
        except Exception:
            exceptions.handle(
                request, _('Unable to delete VPN service.'))


class DeleteEndpointGroupLink(policy.PolicyTargetMixin, tables.DeleteAction):
    name = "deleteendpointgroup"
    policy_rules = (("network", "delete_endpointgroup"),)

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Delete Endpoint Group",
            "Delete Endpoint Groups",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Scheduled deletion of endpoint group",
            "Scheduled deletion of endpoint groups",
            count
        )

    def delete(self, request, obj_id):
        try:
            api_vpn.endpointgroup_delete(request, obj_id)
        except Exception:
            exceptions.handle(
                request, _('Unable to delete endpoint group.'))


class DeleteIKEPolicyLink(policy.PolicyTargetMixin, tables.DeleteAction):
    name = "deleteikepolicy"
    policy_rules = (("network", "delete_ikepolicy"),)

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Delete IKE Policy",
            "Delete IKE Policies",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Scheduled deletion of IKE policy",
            "Scheduled deletion of IKE policies",
            count
        )

    def allowed(self, request, datum=None):
        if datum and datum.ipsecsiteconns:
            return False
        return True

    def delete(self, request, obj_id):
        try:
            api_vpn.ikepolicy_delete(request, obj_id)
        except Exception:
            exceptions.handle(
                request, _('Unable to delete IKE policy.'))


class DeleteIPsecPolicyLink(policy.PolicyTargetMixin, tables.DeleteAction):
    name = "deleteipsecpolicy"
    policy_rules = (("network", "delete_ipsecpolicy"),)

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Delete IPsec Policy",
            "Delete IPsec Policies",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Scheduled deletion of IPsec policy",
            "Scheduled deletion of IPsec policies",
            count
        )

    def allowed(self, request, datum=None):
        if datum and datum.ipsecsiteconns:
            return False
        return True

    def delete(self, request, obj_id):
        try:
            api_vpn.ipsecpolicy_delete(request, obj_id)
        except Exception:
            exceptions.handle(
                request, _('Unable to delete IPsec policy.'))


class DeleteIPsecSiteConnectionLink(policy.PolicyTargetMixin,
                                    tables.DeleteAction):
    name = "deleteipsecsiteconnection"
    policy_rules = (("network", "delete_ipsec_site_connection"),)

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Delete IPsec Site Connection",
            "Delete IPsec Site Connections",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Scheduled deletion of IPsec site connection",
            "Scheduled deletion of IPsec site connections",
            count
        )

    def delete(self, request, obj_id):
        try:
            api_vpn.ipsecsiteconnection_delete(request, obj_id)
        except Exception:
            exceptions.handle(request,
                              _('Unable to delete IPsec site connection.'))


class UpdateVPNServiceLink(tables.LinkAction):
    name = "update_vpnservice"
    verbose_name = _("Edit VPN Service")
    classes = ("ajax-modal", "btn-update",)
    policy_rules = (("network", "update_vpnservice"),)

    def get_link_url(self, vpnservice):
        return reverse("horizon:project:vpn:update_vpnservice",
                       kwargs={'vpnservice_id': vpnservice.id})

    def allowed(self, request, datum=None):
        if datum and datum.status not in forbid_updates:
            return True
        return False


class UpdateEndpointGroupLink(tables.LinkAction):
    name = "updateendpointgroup"
    verbose_name = _("Edit Endpoint Group")
    classes = ("ajax-modal", "btn-update",)
    policy_rules = (("network", "update_endpointgroup"),)

    def get_link_url(self, endpoint_group):
        return reverse("horizon:project:vpn:update_endpointgroup",
                       kwargs={'endpoint_group_id': endpoint_group.id})


class UpdateIKEPolicyLink(tables.LinkAction):
    name = "updateikepolicy"
    verbose_name = _("Edit IKE Policy")
    classes = ("ajax-modal", "btn-update",)
    policy_rules = (("network", "update_ikepolicy"),)

    def get_link_url(self, ikepolicy):
        return reverse("horizon:project:vpn:update_ikepolicy",
                       kwargs={'ikepolicy_id': ikepolicy.id})

    def allowed(self, request, datum=None):
        return not datum['ipsecsiteconns']


class UpdateIPsecPolicyLink(tables.LinkAction):
    name = "updateipsecpolicy"
    verbose_name = _("Edit IPsec Policy")
    classes = ("ajax-modal", "btn-update",)
    policy_rules = (("network", "update_ipsecpolicy"),)

    def get_link_url(self, ipsecpolicy):
        return reverse("horizon:project:vpn:update_ipsecpolicy",
                       kwargs={'ipsecpolicy_id': ipsecpolicy.id})

    def allowed(self, request, datum=None):
        return not datum['ipsecsiteconns']


class UpdateIPsecSiteConnectionLink(tables.LinkAction):
    name = "updateipsecsiteconnection"
    verbose_name = _("Edit Connection")
    classes = ("ajax-modal", "btn-update",)
    policy_rules = (("network", "update_ipsec_site_connection"),)

    def get_link_url(self, ipsecsiteconnection):
        return reverse("horizon:project:vpn:update_ipsecsiteconnection",
                       kwargs={'ipsecsiteconnection_id':
                               ipsecsiteconnection.id})

    def allowed(self, request, datum=None):
        if datum and datum.status not in forbid_updates:
            return True
        return False


STATUS_CHOICES = (
    ("active", True),
    ("down", True),
    ("created", True),
    ("error", False),
    ("inactive", False),
)


STATUS_DISPLAY_CHOICES = (
    ("active", pgettext_lazy("Current status of an IPsec site connection "
                             "and VPN service",
                             "Active")),
    ("down", pgettext_lazy("Current status of an IPsec site connection "
                           "and VPN service",
                           "Down")),
    ("error", pgettext_lazy("Current status of an IPsec site connection "
                            "and VPN service",
                            "Error")),
    ("created", pgettext_lazy("Current status of an IPsec site connection "
                              "and VPN service",
                              "Created")),
    ("pending_create", pgettext_lazy("Current status of an "
                                     "IPsec site connection and VPN service",
                                     "Pending Create")),
    ("pending_update", pgettext_lazy("Current status of an "
                                     "IPsec site connection and VPN service",
                                     "Pending Update")),
    ("pending_delete", pgettext_lazy("Current status of an "
                                     "IPsec site connection and VPN service",
                                     "Pending Delete")),
    ("inactive", pgettext_lazy("Current status of an IPsec site connection "
                               "and VPN service",
                               "Inactive")),
)


class UpdateIPsecSiteConnectionRow(tables.Row):
    ajax = True

    def get_data(self, request, conn_id):
        conn = api_vpn.ipsecsiteconnection_get(request, conn_id)
        conn.ikepolicy_name = conn['ikepolicy'].get('name',
                                                    conn['ikepolicy_id'])
        conn.ipsecpolicy_name = conn['ipsecpolicy'].get('name',
                                                        conn['ipsecpolicy_id'])
        conn.vpnservice_name = conn['vpnservice'].get('name',
                                                      conn['vpnservice_id'])
        return conn


class IPSSCFilterAction(tables.FilterAction):
    name = 'IPSSC_project_IKEPolicies'
    filter_type = 'server'
    filter_choices = (
        ('name', _("Name ="), True),
        ('vpnservice', _("VPN Service ="), True),
        ('vpnservice_id', _("VPN Service ID ="), True),
        ('ikepolicy', _("IKE Policy ="), True),
        ('ikepolicy_id', _("IKE Policy ID ="), True),
        ('ipsecpolicy', _("IPsec Policy ="), True),
        ('ipsecpolicy_id', _("IPsec Policy ID ="), True),
        ('status', _("Status ="), True)
    )


class IPsecSiteConnectionsTable(tables.DataTable):
    name = tables.Column('name_or_id', verbose_name=_('Name'),
                         link="horizon:project:vpn:ipsecsiteconnectiondetails")
    description = tables.Column('description', verbose_name=_('Description'))
    vpnservice_name = tables.Column('vpnservice_name',
                                    verbose_name=_('VPN Service'))
    ikepolicy_name = tables.Column('ikepolicy_name',
                                   verbose_name=_('IKE Policy'))
    ipsecpolicy_name = tables.Column('ipsecpolicy_name',
                                     verbose_name=_('IPsec Policy'))
    status = tables.Column("status",
                           verbose_name=_("Status"),
                           status=True,
                           status_choices=STATUS_CHOICES,
                           display_choices=STATUS_DISPLAY_CHOICES)

    def get_object_display(self, ipsecsiteconnection):
        return ipsecsiteconnection.name_or_id

    class Meta(object):
        name = "ipsecsiteconnectionstable"
        verbose_name = _("IPsec Site Connections")
        status_columns = ['status']
        row_class = UpdateIPsecSiteConnectionRow
        table_actions = (AddIPsecSiteConnectionLink,
                         DeleteIPsecSiteConnectionLink,
                         IPSSCFilterAction)
        row_actions = (UpdateIPsecSiteConnectionLink,
                       DeleteIPsecSiteConnectionLink)


def get_local_ips(vpn):
    template_name = 'project/vpn/_vpn_ips.html'
    context = {"external_v4_ip": vpn.get('external_v4_ip'),
               "external_v6_ip": vpn.get('external_v6_ip')}
    return template.loader.render_to_string(template_name, context)


def get_subnet_name(vpn):
    try:
        return vpn.subnet_name
    except AttributeError:
        return _("-")


class UpdateVPNServiceRow(tables.Row):
    ajax = True

    def get_data(self, request, vpn_id):
        vpn = api_vpn.vpnservice_get(request, vpn_id)
        vpn.router_name = vpn['router'].get('name', vpn['router_id'])
        if 'subnet' in vpn:
            vpn.subnet_name = vpn['subnet'].get('cidr', vpn['subnet_id'])
        return vpn


class VPNServicesFilterAction(tables.FilterAction):
    name = 'vpnservices_project_IKEPolicies'
    filter_type = 'server'
    filter_choices = (
        ('name', _("Name ="), True),
        ('subnet_id', _("Subnet ID ="), True),
        ('subnet_name', _("Subnet ="), True),
        ('router_id', _("Router ID ="), True),
        ('router_name', _("Router ="), True),
    )


class VPNServicesTable(tables.DataTable):
    name = tables.Column("name_or_id", verbose_name=_('Name'),
                         link="horizon:project:vpn:vpnservicedetails")
    description = tables.Column('description', verbose_name=_('Description'))
    local_ips = tables.Column(get_local_ips,
                              verbose_name=_("Local Side Public IPs"))
    subnet_name = tables.Column(get_subnet_name, verbose_name=_('Subnet'))
    router_name = tables.Column('router_name', verbose_name=_('Router'))
    status = tables.Column("status",
                           verbose_name=_("Status"))

    def get_object_display(self, vpnservice):
        return vpnservice.name_or_id

    class Meta(object):
        name = "vpnservicestable"
        verbose_name = _("VPN Services")
        status_columns = ['status']
        row_class = UpdateVPNServiceRow
        table_actions = (AddVPNServiceLink,
                         DeleteVPNServiceLink,
                         VPNServicesFilterAction)
        row_actions = (UpdateVPNServiceLink, DeleteVPNServiceLink)


class EndpointGroupFilterAction(tables.FilterAction):
    name = 'endpointgroups_project'
    filter_type = 'server'
    filter_choices = (
        ('name', _("Name ="), True),
        ('type', _("Type ="), True),
        ('endpoints', _("Endpoints ="), True),
    )


def _get_endpoints(epg):
    return ', '.join(epg.endpoints)


class EndpointGroupTable(tables.DataTable):
    name = tables.Column("name_or_id", verbose_name=_('Name'),
                         link="horizon:project:vpn:endpointgroupdetails")
    description = tables.Column('description', verbose_name=_('Description'))
    type = tables.Column('type', verbose_name=_('Type'))
    endpoints = tables.Column(_get_endpoints, verbose_name=_('Endpoints'))

    class Meta(object):
        name = "endpointgroupstable"
        verbose_name = _("Endpoint Groups")
        table_actions = (AddEndpointGroupLink,
                         DeleteEndpointGroupLink,
                         EndpointGroupFilterAction)
        row_actions = (UpdateEndpointGroupLink, DeleteEndpointGroupLink)

    def get_object_display(self, endpoitgroup):
        return endpoitgroup.name_or_id


class PoliciesFilterAction(tables.FilterAction):
    name = 'filter_project_IKEPolicies'
    filter_type = 'server'
    filter_choices = (
        ('name', _("Name ="), True),
        ('auth_algorithm', _("Authorization algorithm ="), True),
        ('encryption_algorithm', _("Encryption algorithm ="), True),
        ('pfs', _("PFS ="), True),
    )


class IKEPoliciesTable(tables.DataTable):
    name = tables.Column("name_or_id", verbose_name=_('Name'),
                         link="horizon:project:vpn:ikepolicydetails")
    description = tables.Column('description', verbose_name=_('Description'))
    auth_algorithm = tables.Column('auth_algorithm',
                                   verbose_name=_('Authorization algorithm'))
    encryption_algorithm = tables.Column(
        'encryption_algorithm',
        verbose_name=_('Encryption algorithm'))
    pfs = tables.Column("pfs", verbose_name=_('PFS'))

    def get_object_display(self, ikepolicy):
        return ikepolicy.name_or_id

    class Meta(object):
        name = "ikepoliciestable"
        verbose_name = _("IKE Policies")
        table_actions = (AddIKEPolicyLink,
                         DeleteIKEPolicyLink,
                         PoliciesFilterAction)
        row_actions = (UpdateIKEPolicyLink, DeleteIKEPolicyLink)


class IPsecPoliciesTable(tables.DataTable):
    name = tables.Column("name_or_id", verbose_name=_('Name'),
                         link="horizon:project:vpn:ipsecpolicydetails")
    description = tables.Column('description', verbose_name=_('Description'))
    auth_algorithm = tables.Column('auth_algorithm',
                                   verbose_name=_('Authorization algorithm'))
    encryption_algorithm = tables.Column(
        'encryption_algorithm',
        verbose_name=_('Encryption algorithm'))
    pfs = tables.Column("pfs", verbose_name=_('PFS'))

    def get_object_display(self, ipsecpolicy):
        return ipsecpolicy.name_or_id

    class Meta(object):
        name = "ipsecpoliciestable"
        verbose_name = _("IPsec Policies")
        table_actions = (AddIPsecPolicyLink,
                         DeleteIPsecPolicyLink,
                         PoliciesFilterAction)
        row_actions = (UpdateIPsecPolicyLink, DeleteIPsecPolicyLink)



# Copyright 2013, Mirantis Inc
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from collections import OrderedDict

from horizon.utils.memoized import memoized
from openstack_dashboard.api import neutron
from openstack_dashboard.contrib.developer.profiler import api as profiler

networkclient = neutron.networkclient


class IKEPolicy(neutron.NeutronAPIDictWrapper):
    """Wrapper for neutron VPN IKE policy."""


class IPsecPolicy(neutron.NeutronAPIDictWrapper):
    """Wrapper for neutron VPN IPsec policy."""


class IPsecSiteConnection(neutron.NeutronAPIDictWrapper):
    """Wrapper for neutron IPsec site connection."""


class VPNService(neutron.NeutronAPIDictWrapper):
    """Wrapper for neutron VPN service."""


class EndpointGroup(neutron.NeutronAPIDictWrapper):
    """Wrapper for neutron Endpoint Group."""


@profiler.trace
def vpnservice_create(request, **kwargs):
    """Create VPN service

    :param request: request context
    :param admin_state_up: admin state (default on)
    :param name: name for VPN service
    :param description: description for VPN service
    :param router_id: router id for router of VPN service
    :param subnet_id: subnet id for subnet of VPN service
    """
    body = {'admin_state_up': kwargs['admin_state_up'],
            'name': kwargs['name'],
            'description': kwargs['description'],
            'router_id': kwargs['router_id']}
    if kwargs.get('subnet_id'):
        body['subnet_id'] = kwargs['subnet_id']
    vpnservice = networkclient(request).create_vpn_service(**body)
    return VPNService(vpnservice.to_dict())


@profiler.trace
def vpnservice_list(request, **kwargs):
    return _vpnservice_list(request, expand_subnet=True,
                            expand_router=True,
                            expand_conns=True, **kwargs)


def _vpnservice_list(request, expand_subnet=False, expand_router=False,
                     expand_conns=False, **kwargs):
    vpnservices = networkclient(request).vpn_services(**kwargs)
    vpns_dicts = [vpn.to_dict() for vpn in vpnservices]
    if expand_subnet:
        subnets = neutron.subnet_list(request)
        subnet_dict = OrderedDict((s.id, s) for s in subnets)
        for s in vpns_dicts:
            if s.get('subnet_id'):
                s['subnet_name'] = subnet_dict.get(s['subnet_id']).cidr
    if expand_router:
        routers = neutron.router_list(request)
        router_dict = OrderedDict((r.id, r) for r in routers)
        for s in vpnservices:
            s['router_name'] = router_dict.get(s['router_id']).name_or_id
    if expand_conns:
        ipsecsiteconns = _ipsecsiteconnection_list(request, **kwargs)
        for s in vpnservices:
            s['ipsecsiteconns'] = [c.id for c in ipsecsiteconns
                                   if c.vpnservice_id == s['id']]
    return [VPNService(v) for v in vpns_dicts]


@profiler.trace
def vpnservice_get(request, vpnservice_id):
    return _vpnservice_get(request, vpnservice_id, expand_subnet=True,
                           expand_router=True, expand_conns=True)


def _vpnservice_get(request, vpnservice_id, expand_subnet=False,
                    expand_router=False, expand_conns=False):
    vpnservice = networkclient(request).get_vpn_service(
        vpnservice_id).to_dict()
    if expand_subnet and ('subnet_id' in vpnservice):
        if vpnservice['subnet_id'] is not None:
            vpnservice['subnet'] = neutron.subnet_get(
                request, vpnservice['subnet_id'])
    if expand_router:
        vpnservice['router'] = neutron.router_get(
            request, vpnservice['router_id'])
    if expand_conns:
        ipsecsiteconns = _ipsecsiteconnection_list(request)
        vpnservice['ipsecsiteconns'] = [c for c in ipsecsiteconns
                                        if c.vpnservice_id == vpnservice['id']]
    return VPNService(vpnservice)


@profiler.trace
def vpnservice_update(request, vpnservice_id, **kwargs):
    vpnservice = networkclient(request).update_vpn_service(
        vpnservice_id, **kwargs).to_dict()
    return VPNService(vpnservice)


@profiler.trace
def vpnservice_delete(request, vpnservice_id):
    networkclient(request).delete_vpn_service(vpnservice_id)


@profiler.trace
def endpointgroup_create(request, **kwargs):
    """Create Endpoint Group

    :param request: request context
    :param name: name for Endpoint Group
    :param description: description for Endpoint Group
    :param type: type of Endpoint Group
    :param endpoints: endpoint(s) of Endpoint Group
    """
    body = {'name': kwargs['name'],
            'description': kwargs['description'],
            'type': kwargs['type'],
            'endpoints': kwargs['endpoints']
            }
    endpointgroup = networkclient(request).create_vpn_endpoint_group(**body)
    return EndpointGroup(endpointgroup.to_dict())


@profiler.trace
def endpointgroup_list(request, **kwargs):
    return _endpointgroup_list(request, expand_conns=True, **kwargs)


def _endpointgroup_list(request, expand_conns=False, **kwargs):
    endpointgroups = networkclient(request).vpn_endpoint_groups(**kwargs)
    vpn_epg_dicts = [epg.to_dict() for epg in endpointgroups]
    if expand_conns:
        ipsecsiteconns = _ipsecsiteconnection_list(request)
        for g in vpn_epg_dicts:
            g['ipsecsiteconns'] = [
                c.id for c in ipsecsiteconns
                if (c.get('local_ep_group_id') == g['id'] or
                    c.get('peer_ep_group_id') == g['id'])]
    return [EndpointGroup(v) for v in vpn_epg_dicts]


@profiler.trace
def endpointgroup_get(request, endpoint_group_id):
    return _endpointgroup_get(request, endpoint_group_id, expand_conns=True)


def _endpointgroup_get(request, endpoint_group_id, expand_conns=False):
    endpointgroup = networkclient(request).get_vpn_endpoint_group(
        endpoint_group_id).to_dict()
    if expand_conns:
        ipsecsiteconns = _ipsecsiteconnection_list(request)
        endpointgroup['ipsecsiteconns'] = [
            c for c in ipsecsiteconns
            if (c.get('local_ep_group_id') == endpointgroup['id'] or
                c.get('peer_ep_group_id') == endpointgroup['id'])]
    return EndpointGroup(endpointgroup)


@profiler.trace
def endpointgroup_update(request, endpoint_group_id, **kwargs):
    endpointgroup = networkclient(request).update_vpn_endpoint_group(
        endpoint_group_id, **kwargs).to_dict()
    return EndpointGroup(endpointgroup)


@profiler.trace
def endpointgroup_delete(request, endpoint_group_id):
    networkclient(request).delete_vpn_endpoint_group(endpoint_group_id)


@profiler.trace
def ikepolicy_create(request, **kwargs):
    """Create IKE policy

    :param request: request context
    :param name: name for IKE policy
    :param description: description for IKE policy
    :param auth_algorithm: authorization algorithm for IKE policy
    :param encryption_algorithm: encryption algorithm for IKE policy
    :param ike_version: IKE version for IKE policy
    :param lifetime: Lifetime Units and Value for IKE policy
    :param pfs: Perfect Forward Secrecy for IKE policy
    :param phase1_negotiation_mode: IKE Phase1 negotiation mode for IKE policy
    """
    body = {'name': kwargs['name'],
            'description': kwargs['description'],
            'auth_algorithm': kwargs['auth_algorithm'],
            'encryption_algorithm': kwargs['encryption_algorithm'],
            'ike_version': kwargs['ike_version'],
            'lifetime': kwargs['lifetime'],
            'pfs': kwargs['pfs'],
            'phase1_negotiation_mode': kwargs['phase1_negotiation_mode']}
    ikepolicy = networkclient(request).create_vpn_ike_policy(**body).to_dict()
    return IKEPolicy(ikepolicy)


@profiler.trace
def ikepolicy_list(request, **kwargs):
    return _ikepolicy_list(request, expand_conns=True, **kwargs)


def _ikepolicy_list(request, expand_conns=False, **kwargs):
    ikepolicies = networkclient(request).vpn_ike_policies(
        **kwargs)
    vpn_ikep_dicts = [ikep.to_dict() for ikep in ikepolicies]
    if expand_conns:
        ipsecsiteconns = _ipsecsiteconnection_list(request, **kwargs)
        for p in vpn_ikep_dicts:
            p['ipsecsiteconns'] = [c.id for c in ipsecsiteconns
                                   if c.ikepolicy_id == p['id']]
    return [IKEPolicy(v) for v in vpn_ikep_dicts]


@profiler.trace
def ikepolicy_get(request, ikepolicy_id):
    return _ikepolicy_get(request, ikepolicy_id, expand_conns=True)


def _ikepolicy_get(request, ikepolicy_id, expand_conns=False):
    ikepolicy = networkclient(request).get_vpn_ike_policy(
        ikepolicy_id).to_dict()
    if expand_conns:
        ipsecsiteconns = _ipsecsiteconnection_list(request)
        ikepolicy['ipsecsiteconns'] = [c for c in ipsecsiteconns
                                       if c.ikepolicy_id == ikepolicy['id']]
    return IKEPolicy(ikepolicy)


@profiler.trace
def ikepolicy_update(request, ikepolicy_id, **kwargs):
    ikepolicy = networkclient(request).update_vpn_ike_policy(
        ikepolicy_id, **kwargs).to_dict()
    return IKEPolicy(ikepolicy)


@profiler.trace
def ikepolicy_delete(request, ikepolicy_id):
    networkclient(request).delete_vpn_ike_policy(ikepolicy_id)


@profiler.trace
def ipsecpolicy_create(request, **kwargs):
    """Create IPsec policy

    :param request: request context
    :param name: name for IPsec policy
    :param description: description for IPsec policy
    :param auth_algorithm: authorization algorithm for IPsec policy
    :param encapsulation_mode: encapsulation mode for IPsec policy
    :param encryption_algorithm: encryption algorithm for IPsec policy
    :param lifetime: Lifetime Units and Value for IPsec policy
    :param pfs: Perfect Forward Secrecy for IPsec policy
    :param transform_protocol: Transform Protocol for IPsec policy
    """
    body = {'name': kwargs['name'],
            'description': kwargs['description'],
            'auth_algorithm': kwargs['auth_algorithm'],
            'encapsulation_mode': kwargs['encapsulation_mode'],
            'encryption_algorithm': kwargs['encryption_algorithm'],
            'lifetime': kwargs['lifetime'],
            'pfs': kwargs['pfs'],
            'transform_protocol': kwargs['transform_protocol']
            }
    ipsecpolicy = networkclient(request).create_vpn_ipsec_policy(**body)
    return IPsecPolicy(ipsecpolicy.to_dict())


@profiler.trace
def ipsecpolicy_list(request, **kwargs):
    return _ipsecpolicy_list(request, expand_conns=True, **kwargs)


def _ipsecpolicy_list(request, expand_conns=False, **kwargs):
    ipsecpolicies = networkclient(request).vpn_ipsec_policies(
        **kwargs)
    ipsec_dicts = [ipsec.to_dict() for ipsec in ipsecpolicies]
    if expand_conns:
        ipsecsiteconns = _ipsecsiteconnection_list(request, **kwargs)
        for p in ipsec_dicts:
            p['ipsecsiteconns'] = [c.id for c in ipsecsiteconns
                                   if c.ipsecpolicy_id == p['id']]
    return [IPsecPolicy(v) for v in ipsec_dicts]


@profiler.trace
def ipsecpolicy_get(request, ipsecpolicy_id):
    return _ipsecpolicy_get(request, ipsecpolicy_id, expand_conns=True)


def _ipsecpolicy_get(request, ipsecpolicy_id, expand_conns=False):
    ipsecpolicy = networkclient(request).get_vpn_ipsec_policy(
        ipsecpolicy_id).to_dict()
    if expand_conns:
        ipsecsiteconns = _ipsecsiteconnection_list(request)
        ipsecpolicy['ipsecsiteconns'] = [c for c in ipsecsiteconns
                                         if (c.ipsecpolicy_id ==
                                             ipsecpolicy['id'])]
    return IPsecPolicy(ipsecpolicy)


@profiler.trace
def ipsecpolicy_update(request, ipsecpolicy_id, **kwargs):
    ipsecpolicy = networkclient(request).update_vpn_ipsec_policy(
        ipsecpolicy_id, **kwargs).to_dict()
    return IPsecPolicy(ipsecpolicy)


@profiler.trace
def ipsecpolicy_delete(request, ipsecpolicy_id):
    networkclient(request).delete_vpn_ipsec_policy(ipsecpolicy_id)


@profiler.trace
def ipsecsiteconnection_create(request, **kwargs):
    """Create IPsec site connection

    :param request: request context
    :param name: name for IPsec site connection
    :param description: description for IPsec site connection
    :param dpd: dead peer detection action, interval and timeout
    :param ikepolicy_id: IKE policy associated with this connection
    :param initiator: initiator state
    :param ipsecpolicy_id: IPsecPolicy associated with this connection
    :param mtu: MTU size for the connection
    :param peer_address: Peer gateway public address
    :param peer_cidrs: remote subnet(s) in CIDR format
    :param peer_id:  Peer router identity for authentication"
    :param psk: Pre-Shared Key string
    :param vpnservice_id: VPN service associated with this connection
    :param admin_state_up: admin state (default on)
    """
    body = {
        'name': kwargs['name'],
        'description': kwargs['description'],
        'dpd': kwargs['dpd'],
        'ikepolicy_id': kwargs['ikepolicy_id'],
        'initiator': kwargs['initiator'],
        'ipsecpolicy_id': kwargs['ipsecpolicy_id'],
        'mtu': kwargs['mtu'],
        'peer_address': kwargs['peer_address'],
        'peer_id': kwargs['peer_id'],
        'psk': kwargs['psk'],
        'vpnservice_id': kwargs['vpnservice_id'],
        'admin_state_up': kwargs['admin_state_up']
    }
    cidrs = kwargs.get('peer_cidrs', [])
    if not cidrs:
        body['local_ep_group_id'] = kwargs['local_ep_group_id']
        body['peer_ep_group_id'] = kwargs['peer_ep_group_id']
    else:
        body['peer_cidrs'] = kwargs['peer_cidrs']
    ipsecsiteconnection = networkclient(
        request).create_vpn_ipsec_site_connection(**body).to_dict()
    return IPsecSiteConnection(ipsecsiteconnection)


@profiler.trace
@memoized
def ipsecsiteconnection_list(request, **kwargs):
    return _ipsecsiteconnection_list(request, expand_ikepolicies=True,
                                     expand_ipsecpolicies=True,
                                     expand_vpnservices=True, **kwargs)


@memoized
def _ipsecsiteconnection_list(request, expand_ikepolicies=False,
                              expand_ipsecpolicies=False,
                              expand_vpnservices=False, **kwargs):
    ipsecsiteconnections = networkclient(request).vpn_ipsec_site_connections(
        **kwargs)
    ipsec_dicts = [ipsec.to_dict() for ipsec in ipsecsiteconnections]
    if expand_ikepolicies:
        ikepolicies = _ikepolicy_list(request)
        policy_dict = OrderedDict((p.id, p) for p in ikepolicies)
        for c in ipsec_dicts:
            c['ikepolicy_name'] = policy_dict.get(c['ikepolicy_id']).name_or_id
    if expand_ipsecpolicies:
        ipsecpolicies = _ipsecpolicy_list(request)
        policy_dict = OrderedDict((p.id, p) for p in ipsecpolicies)
        for c in ipsec_dicts:
            c['ipsecpolicy_name'] = policy_dict.get(c['ipsecpolicy_id']
                                                    ).name_or_id
    if expand_vpnservices:
        vpnservices = _vpnservice_list(request)
        service_dict = OrderedDict((s.id, s) for s in vpnservices)
        for c in ipsec_dicts:
            c['vpnservice_name'] = service_dict.get(c['vpnservice_id']
                                                    ).name_or_id
    return [IPsecSiteConnection(v) for v in ipsec_dicts]


@profiler.trace
def ipsecsiteconnection_get(request, ipsecsiteconnection_id):
    return _ipsecsiteconnection_get(request, ipsecsiteconnection_id,
                                    expand_ikepolicies=True,
                                    expand_ipsecpolicies=True,
                                    expand_vpnservices=True)


def _ipsecsiteconnection_get(request, ipsecsiteconnection_id,
                             expand_ikepolicies, expand_ipsecpolicies,
                             expand_vpnservices):
    ipsecsiteconnection = networkclient(
        request).get_vpn_ipsec_site_connection(
            ipsecsiteconnection_id).to_dict()
    if expand_ikepolicies:
        ipsecsiteconnection['ikepolicy'] = _ikepolicy_get(
            request, ipsecsiteconnection['ikepolicy_id'])
    if expand_ipsecpolicies:
        ipsecsiteconnection['ipsecpolicy'] = _ipsecpolicy_get(
            request, ipsecsiteconnection['ipsecpolicy_id'])
    if expand_vpnservices:
        ipsecsiteconnection['vpnservice'] = _vpnservice_get(
            request, ipsecsiteconnection['vpnservice_id'])
    return IPsecSiteConnection(ipsecsiteconnection)


@profiler.trace
def ipsecsiteconnection_update(request, ipsecsiteconnection_id, **kwargs):
    ipsecsiteconnection = networkclient(
        request).update_vpn_ipsec_site_connection(
            ipsecsiteconnection_id, **kwargs).to_dict()
    return IPsecSiteConnection(ipsecsiteconnection)


@profiler.trace
def ipsecsiteconnection_delete(request, ipsecsiteconnection_id):
    networkclient(request).delete_vpn_ipsec_site_connection(
        ipsecsiteconnection_id)



# Copyright 2013, Mirantis Inc
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import forms as horizon_forms
from horizon import tabs as horizon_tabs
from horizon.utils import memoized
from horizon import workflows as horizon_workflows

from neutron_vpnaas_dashboard.api import vpn as api_vpn
from neutron_vpnaas_dashboard.dashboards.project.vpn import forms
from neutron_vpnaas_dashboard.dashboards.project.vpn import tables
from neutron_vpnaas_dashboard.dashboards.project.vpn import tabs
from neutron_vpnaas_dashboard.dashboards.project.vpn import workflows


class IndexView(horizon_tabs.TabbedTableView):
    tab_group_class = tabs.VPNTabs
    template_name = 'project/vpn/index.html'
    page_title = _("Virtual Private Network")


class AddVPNServiceView(horizon_workflows.WorkflowView):
    workflow_class = workflows.AddVPNService


class AddEndpointGroupView(horizon_workflows.WorkflowView):
    workflow_class = workflows.AddEndpointGroup


class AddIPsecSiteConnectionView(horizon_workflows.WorkflowView):
    workflow_class = workflows.AddIPsecSiteConnection


class AddIKEPolicyView(horizon_workflows.WorkflowView):
    workflow_class = workflows.AddIKEPolicy


class AddIPsecPolicyView(horizon_workflows.WorkflowView):
    workflow_class = workflows.AddIPsecPolicy


class IKEPolicyDetailsView(horizon_tabs.TabView):
    tab_group_class = tabs.IKEPolicyDetailsTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ ikepolicy.name|default:ikepolicy.id }}"

    @memoized.memoized_method
    def get_data(self):
        pid = self.kwargs['ikepolicy_id']
        try:
            return api_vpn.ikepolicy_get(self.request, pid)
        except Exception:
            msg = _('Unable to retrieve IKE policy details.')
            exceptions.handle(self.request, msg,
                              redirect=self.get_redirect_url())

    def get_context_data(self, **kwargs):
        context = super(IKEPolicyDetailsView, self).get_context_data(**kwargs)
        ikepolicy = self.get_data()
        table = tables.IKEPoliciesTable(self.request)
        context["ikepolicy"] = ikepolicy
        context["url"] = self.get_redirect_url()
        context["actions"] = table.render_row_actions(ikepolicy)
        return context

    def get_tabs(self, request, *args, **kwargs):
        ikepolicy = self.get_data()
        return self.tab_group_class(request, ikepolicy=ikepolicy, **kwargs)

    @staticmethod
    def get_redirect_url():
        return reverse('horizon:project:vpn:index')


class IPsecPolicyDetailsView(horizon_tabs.TabView):
    tab_group_class = tabs.IPsecPolicyDetailsTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ ipsecpolicy.name|default:ipsecpolicy.id }}"

    @memoized.memoized_method
    def get_data(self):
        pid = self.kwargs['ipsecpolicy_id']
        try:
            return api_vpn.ipsecpolicy_get(self.request, pid)
        except Exception:
            msg = _('Unable to retrieve IPsec policy details.')
            exceptions.handle(self.request, msg,
                              redirect=self.get_redirect_url())

    def get_context_data(self, **kwargs):
        context = super(IPsecPolicyDetailsView, self).get_context_data(
            **kwargs)
        ipsecpolicy = self.get_data()
        table = tables.IPsecPoliciesTable(self.request)
        context["ipsecpolicy"] = ipsecpolicy
        context["url"] = self.get_redirect_url()
        context["actions"] = table.render_row_actions(ipsecpolicy)
        return context

    def get_tabs(self, request, *args, **kwargs):
        ipsecpolicy = self.get_data()
        return self.tab_group_class(request, ipsecpolicy=ipsecpolicy, **kwargs)

    @staticmethod
    def get_redirect_url():
        return reverse('horizon:project:vpn:index')


class VPNServiceDetailsView(horizon_tabs.TabView):
    tab_group_class = tabs.VPNServiceDetailsTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ vpnservice.name|default:vpnservice.id }}"

    @memoized.memoized_method
    def get_data(self):
        sid = self.kwargs['vpnservice_id']

        try:
            vpnservice = api_vpn.vpnservice_get(self.request, sid)
        except Exception:
            vpnservice = []
            msg = _('Unable to retrieve VPN service details.')
            exceptions.handle(self.request, msg,
                              redirect=self.get_redirect_url())
        try:
            connections = api_vpn.ipsecsiteconnection_list(
                self.request, vpnservice_id=sid)
            vpnservice.vpnconnections = connections
        except Exception:
            vpnservice.vpnconnections = []

        return vpnservice

    def get_context_data(self, **kwargs):
        context = super(VPNServiceDetailsView, self).get_context_data(**kwargs)
        vpnservice = self.get_data()
        table = tables.VPNServicesTable(self.request)
        context["vpnservice"] = vpnservice
        context["url"] = self.get_redirect_url()
        context["actions"] = table.render_row_actions(vpnservice)
        return context

    def get_tabs(self, request, *args, **kwargs):
        vpnservice = self.get_data()
        return self.tab_group_class(request, vpnservice=vpnservice, **kwargs)

    @staticmethod
    def get_redirect_url():
        return reverse('horizon:project:vpn:index')


class EndpointGroupDetailsView(horizon_tabs.TabView):
    tab_group_class = tabs.EndpointGroupDetailsTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ endpointgroup.name|default:endpointgroup.id }}"

    @memoized.memoized_method
    def get_data(self):
        gid = self.kwargs['endpoint_group_id']

        try:
            endpointgroup = api_vpn.endpointgroup_get(self.request, gid)
        except Exception:
            msg = _('Unable to retrieve endpoint group details.')
            exceptions.handle(self.request, msg,
                              redirect=self.get_redirect_url())
        try:
            connections = api_vpn.ipsecsiteconnection_list(
                self.request, endpoint_group_id=gid)
            endpointgroup.vpnconnections = connections
        except Exception:
            endpointgroup.vpnconnections = []

        return endpointgroup

    def get_context_data(self, **kwargs):
        context = super(EndpointGroupDetailsView, self).get_context_data(
            **kwargs)
        endpointgroup = self.get_data()
        table = tables.EndpointGroupTable(self.request)
        context["endpointgroup"] = endpointgroup
        context["url"] = self.get_redirect_url()
        context["actions"] = table.render_row_actions(endpointgroup)
        return context

    def get_tabs(self, request, *args, **kwargs):
        endpointgroup = self.get_data()
        return self.tab_group_class(request, endpointgroup=endpointgroup,
                                    **kwargs)

    @staticmethod
    def get_redirect_url():
        return reverse('horizon:project:vpn:index')


class IPsecSiteConnectionDetailsView(horizon_tabs.TabView):
    tab_group_class = tabs.IPsecSiteConnectionDetailsTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ ipsecsiteconnection.name|default:ipsecsiteconnection.id}}"

    @memoized.memoized_method
    def get_data(self):
        cid = self.kwargs['ipsecsiteconnection_id']
        try:
            return api_vpn.ipsecsiteconnection_get(self.request, cid)
        except Exception:
            msg = _('Unable to retrieve IPsec site connection details.')
            exceptions.handle(self.request, msg,
                              redirect=self.get_redirect_url())

    def get_context_data(self, **kwargs):
        context = super(IPsecSiteConnectionDetailsView, self).get_context_data(
            **kwargs)
        ipsecsiteconnection = self.get_data()
        table = tables.IPsecSiteConnectionsTable(self.request)
        context["ipsecsiteconnection"] = ipsecsiteconnection
        context["url"] = self.get_redirect_url()
        context["actions"] = table.render_row_actions(ipsecsiteconnection)
        return context

    def get_tabs(self, request, *args, **kwargs):
        ipsecsiteconnection = self.get_data()
        return self.tab_group_class(request,
                                    ipsecsiteconnection=ipsecsiteconnection,
                                    **kwargs)

    @staticmethod
    def get_redirect_url():
        return reverse('horizon:project:vpn:index')


class UpdateVPNServiceView(horizon_forms.ModalFormView):
    form_class = forms.UpdateVPNService
    form_id = "update_vpnservice_form"
    template_name = "project/vpn/update_vpnservice.html"
    context_object_name = 'vpnservice'
    submit_label = _("Save Changes")
    submit_url = "horizon:project:vpn:update_vpnservice"
    success_url = reverse_lazy("horizon:project:vpn:index")
    page_title = _("Edit VPN Service")

    def get_context_data(self, **kwargs):
        context = super(UpdateVPNServiceView, self).get_context_data(**kwargs)
        context["vpnservice_id"] = self.kwargs['vpnservice_id']
        args = (self.kwargs['vpnservice_id'],)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    @memoized.memoized_method
    def _get_object(self, *args, **kwargs):
        vpnservice_id = self.kwargs['vpnservice_id']
        try:
            return api_vpn.vpnservice_get(self.request, vpnservice_id)
        except Exception:
            redirect = self.success_url
            msg = _('Unable to retrieve VPN service details.')
            exceptions.handle(self.request, msg, redirect=redirect)

    def get_initial(self):
        vpnservice = self._get_object()
        return {'name': vpnservice['name'],
                'vpnservice_id': vpnservice['id'],
                'description': vpnservice['description'],
                'admin_state_up': vpnservice['admin_state_up']}


class UpdateEndpointGroupView(horizon_forms.ModalFormView):
    form_class = forms.UpdateEndpointGroup
    form_id = "update_endpointgroup_form"
    template_name = "project/vpn/update_endpointgroup.html"
    context_object_name = 'endpointgroup'
    submit_label = _("Save Changes")
    submit_url = "horizon:project:vpn:update_endpointgroup"
    success_url = reverse_lazy("horizon:project:vpn:index")
    page_title = _("Edit Endpoint Group")

    def get_context_data(self, **kwargs):
        context = super(UpdateEndpointGroupView, self).get_context_data(
            **kwargs)
        context["endpoint_group_id"] = self.kwargs['endpoint_group_id']
        args = (self.kwargs['endpoint_group_id'],)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    @memoized.memoized_method
    def _get_object(self, *args, **kwargs):
        endpoint_group_id = self.kwargs['endpoint_group_id']
        try:
            return api_vpn.endpointgroup_get(self.request, endpoint_group_id)
        except Exception:
            redirect = self.success_url
            msg = _('Unable to retrieve endpoint group details.')
            exceptions.handle(self.request, msg, redirect=redirect)

    def get_initial(self):
        endpointgroup = self._get_object()
        return {'name': endpointgroup['name'],
                'endpoint_group_id': endpointgroup['id'],
                'description': endpointgroup['description']}


class UpdateIKEPolicyView(horizon_forms.ModalFormView):
    form_class = forms.UpdateIKEPolicy
    form_id = "update_ikepolicy_form"
    template_name = "project/vpn/update_ikepolicy.html"
    context_object_name = 'ikepolicy'
    submit_label = _("Save Changes")
    submit_url = "horizon:project:vpn:update_ikepolicy"
    success_url = reverse_lazy("horizon:project:vpn:index")
    page_title = _("Edit IKE Policy")

    def get_context_data(self, **kwargs):
        context = super(UpdateIKEPolicyView, self).get_context_data(**kwargs)
        context["ikepolicy_id"] = self.kwargs['ikepolicy_id']
        args = (self.kwargs['ikepolicy_id'],)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    @memoized.memoized_method
    def _get_object(self, *args, **kwargs):
        ikepolicy_id = self.kwargs['ikepolicy_id']
        try:
            return api_vpn.ikepolicy_get(self.request, ikepolicy_id)
        except Exception:
            redirect = self.success_url
            msg = _('Unable to retrieve IKE policy details.')
            exceptions.handle(self.request, msg, redirect=redirect)

    def get_initial(self):
        ikepolicy = self._get_object()
        return {'name': ikepolicy['name'],
                'ikepolicy_id': ikepolicy['id'],
                'description': ikepolicy['description'],
                'auth_algorithm': ikepolicy['auth_algorithm'],
                'encryption_algorithm': ikepolicy['encryption_algorithm'],
                'ike_version': ikepolicy['ike_version'],
                'lifetime_units': ikepolicy['lifetime']['units'],
                'lifetime_value': ikepolicy['lifetime']['value'],
                'pfs': ikepolicy['pfs'],
                'phase1_negotiation_mode': ikepolicy[
                    'phase1_negotiation_mode']}


class UpdateIPsecPolicyView(horizon_forms.ModalFormView):
    form_class = forms.UpdateIPsecPolicy
    form_id = "update_ipsecpolicy_form"
    template_name = "project/vpn/update_ipsecpolicy.html"
    context_object_name = 'ipsecpolicy'
    submit_label = _("Save Changes")
    submit_url = "horizon:project:vpn:update_ipsecpolicy"
    success_url = reverse_lazy("horizon:project:vpn:index")
    page_title = _("Edit IPsec Policy")

    def get_context_data(self, **kwargs):
        context = super(UpdateIPsecPolicyView, self).get_context_data(**kwargs)
        context["ipsecpolicy_id"] = self.kwargs['ipsecpolicy_id']
        args = (self.kwargs['ipsecpolicy_id'],)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    @memoized.memoized_method
    def _get_object(self, *args, **kwargs):
        ipsecpolicy_id = self.kwargs['ipsecpolicy_id']
        try:
            return api_vpn.ipsecpolicy_get(self.request, ipsecpolicy_id)
        except Exception:
            redirect = self.success_url
            msg = _('Unable to retrieve IPsec policy details.')
            exceptions.handle(self.request, msg, redirect=redirect)

    def get_initial(self):
        ipsecpolicy = self._get_object()
        return {'name': ipsecpolicy['name'],
                'ipsecpolicy_id': ipsecpolicy['id'],
                'description': ipsecpolicy['description'],
                'auth_algorithm': ipsecpolicy['auth_algorithm'],
                'encapsulation_mode': ipsecpolicy['encapsulation_mode'],
                'encryption_algorithm': ipsecpolicy['encryption_algorithm'],
                'lifetime_units': ipsecpolicy['lifetime']['units'],
                'lifetime_value': ipsecpolicy['lifetime']['value'],
                'pfs': ipsecpolicy['pfs'],
                'transform_protocol': ipsecpolicy['transform_protocol']}


class UpdateIPsecSiteConnectionView(horizon_forms.ModalFormView):
    form_class = forms.UpdateIPsecSiteConnection
    form_id = "update_ipsecsiteconnection_form"
    template_name = "project/vpn/update_ipsecsiteconnection.html"
    context_object_name = 'ipsecsiteconnection'
    submit_label = _("Save Changes")
    submit_url = "horizon:project:vpn:update_ipsecsiteconnection"
    success_url = reverse_lazy("horizon:project:vpn:index")
    page_title = _("Edit IPsec Site Connection")

    def get_context_data(self, **kwargs):
        context = super(
            UpdateIPsecSiteConnectionView, self).get_context_data(**kwargs)
        context["ipsecsiteconnection_id"] = self.kwargs[
            'ipsecsiteconnection_id']
        args = (self.kwargs['ipsecsiteconnection_id'],)
        context['submit_url'] = reverse(self.submit_url, args=args)
        return context

    @memoized.memoized_method
    def _get_object(self, *args, **kwargs):
        connection_id = self.kwargs['ipsecsiteconnection_id']
        try:
            return api_vpn.ipsecsiteconnection_get(self.request, connection_id)
        except Exception:
            redirect = self.success_url
            msg = _('Unable to retrieve IPsec site connection details.')
            exceptions.handle(self.request, msg, redirect=redirect)

    def get_initial(self):
        ipsecsiteconnection = self._get_object()
        data = {
            'name': ipsecsiteconnection['name'],
            'ipsecsiteconnection_id': ipsecsiteconnection['id'],
            'description': ipsecsiteconnection['description'],
            'peer_address': ipsecsiteconnection['peer_address'],
            'peer_id': ipsecsiteconnection['peer_id'],
            'psk': ipsecsiteconnection['psk'],
            'mtu': ipsecsiteconnection['mtu'],
            'dpd_action': ipsecsiteconnection['dpd']['action'],
            'dpd_interval': ipsecsiteconnection['dpd']['interval'],
            'dpd_timeout': ipsecsiteconnection['dpd']['timeout'],
            'initiator': ipsecsiteconnection['initiator'],
            'admin_state_up': ipsecsiteconnection['admin_state_up']
        }
        if 'local_ep_group_id' in ipsecsiteconnection:
            data['local_ep_group_id'] = \
                ipsecsiteconnection['local_ep_group_id']
            data['peer_ep_group_id'] = ipsecsiteconnection['peer_ep_group_id']
            return data
        else:
            data['peer_cidrs'] = ", ".join(ipsecsiteconnection['peer_cidrs'])
            return data
